/*
    AttendanceManager
    Copyright (C) 2010  Victor Carceler

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>
 */

package org.iespuigcastellar.attendancemanager.core;

import java.sql.Time;
import java.util.Date;

/**
 * Model a database classblock record. Has all record columns plus a name
 * property used by the user interface. Name is not stored on DBMS.
 * 
 * @author vcarceler
 */
public class Classblock implements HasGetName {


	private int id;
	private int idsubject;
	private int idgroup;
	private int idterm;
	private int idteacher;
	private int day_of_week;
	private Time start;
	private Time end;
	private String name; // It's a name composed by time + group.name + " / " + subject.name
	

	public Classblock(int id, int idsubject, int idgroup, int day_of_week, Time start, Time end) {
		this.id = id;
		this.idsubject = idsubject;
		this.idgroup = idgroup;
		this.day_of_week = day_of_week;
		this.start = start;
		this.end = end;
	}

	public Classblock(int id,int idterm,int idteacher, int idsubject, int idgroup, int day_of_week, Time start, Time end) {
		this(id, idsubject, idgroup, day_of_week, start, end);

		this.idterm=idterm;
		this.idteacher=idteacher;		
	}

	public int getIdgroup() {
		return idgroup;
	}

	public void setIdgroup(int idgroup) {
		this.idgroup = idgroup;
	}

	public int getIdterm() {
		return idterm;
	}

	public void setIdterm(int idterm) {
		this.idterm = idterm;
	}

	public int getIdteacher() {
		return idteacher;
	}

	public void setIdteacher(int idteacher) {
		this.idteacher = idteacher;
	}
	public int getID() {
		return id;
	}
	public void setID(int id) {
		this.id = id;
	}
	public int getIDSubject() {
		return idsubject;
	}
	public void setIDSubject(int idsubject) {
		this.idsubject = idsubject;
	}
	public int getDay_of_week() {
		return day_of_week;
	}
	public void setDay_of_week(int dayOfWeek) {
		day_of_week = dayOfWeek;
	}
	public Time getStart() {
		return start;
	}
	public void setStart(Time start) {
		this.start = start;
	}
	public Time getEnd() {
		return end;
	}
	public void setEnd(Time end) {
		this.end = end;
	}

	public void setIDGroup(int idgroup) {
		this.idgroup = idgroup;
	}

	public int getIDGroup() {
		return idgroup;
	}

	public void setName(String name) {
		this.name = name;
	}

	public String getName() {
		return name;
	}

	@Override
	public String toString() {
		return "Classblock [day_of_week=" + day_of_week + ", end=" + end
				+ ", id=" + id + ", idgroup=" + idgroup + ", idsubject="
				+ idsubject + ", name=" + name + ", start=" + start + "]";
	}
}
