/*
    AttendanceManager
    Copyright (C) 2010  Victor Carceler

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>
 */

package org.iespuigcastellar.attendancemanager.core;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.nio.charset.Charset;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.text.DateFormat;
import java.text.ParseException;
import java.util.Calendar;
import java.util.Date;
import java.util.Locale;
import java.util.ResourceBundle;
import java.util.Vector;

import com.csvreader.CsvWriter;
import com.itextpdf.text.pdf.PdfPTable;

/**
 * Esta clase permite generar informes en formato CSV. Los informes se
 * escriben en un <code>OutputStream</code> que proporciona el usuario.
 *
 * 
 *
 * @author dperez David Pérez <dperez@iespuigcastellar.xeill.net>
 *
 */
public class CreateCSV {

    private DatabaseStorage database= new DatabaseStorage();
    private CsvWriter csv;
    ResourceBundle rb = ResourceBundle.getBundle("AttendanceManager");

    /**
     * Generate type 1 reports, un listado de estudiantes del grupo con sus
     * correspondientes faltas.
     *
     * @param    idgroup    Identificador del grupo para genera el informe
     * @param    dateA    Cadena que especifica la fecha de origen para el informe
     * en formato SQL (YYYY-MM-DD)
     * @param     dateB    Cadena que especifica la fecha final para el informe
     * @throws ParseException
     */
    public ByteArrayOutputStream createReport1(int idgroup, Date dfecha1, Date dfecha2) throws SQLException, IOException, ParseException{
       
        ByteArrayOutputStream ByteArrayOutputStreamCSV=new ByteArrayOutputStream();
       
        this.csv = new CsvWriter(ByteArrayOutputStreamCSV, ',', Charset.forName("UTF-8"));
        this.csv.setTextQualifier('\"');
        this.csv.setForceQualifier(true);
        String output_headers[] = {"surname1", "surname2", "name", "misscount", "excussedcount", "delaycount", "expulsioncount"};
        this.csv.writeRecord(output_headers);
       
        ResultSet result = database.studentReport1(idgroup,dfecha1, dfecha2);
       
        while (result.next()) {
           
            String nombre= result.getString("name");
            String apellido1= result.getString("surname1");
            String apellido2= result.getString("surname2");
            int miss= result.getInt("misscount");
            int excused= result.getInt("excusedcount");
            int delay= result.getInt("delaycount");
            int expulsion= result.getInt("expulsioncount");
           
            String record[] = {apellido1,apellido2,nombre,""+miss+"",""+excused+"",""+delay+"",""+expulsion+""};
            csv.writeRecord(record);
        }
        csv.close();
        return ByteArrayOutputStreamCSV;
    }
   
    public ByteArrayOutputStream createReport2(int idgroup, Date dfecha1, Date dfecha2) throws SQLException, IOException, ParseException{
       
       
        ByteArrayOutputStream ByteArrayOutputStreamCSV= new ByteArrayOutputStream();
       
       
        this.csv = new CsvWriter(ByteArrayOutputStreamCSV, ',', Charset.forName("UTF-8"));
        this.csv.setTextQualifier('\"');
        this.csv.setForceQualifier(true);
        String output_headers[] = {"surname1", "surname2", "name", "subject", "misscount", "excussedcount", "delaycount", "expulsioncount"};
        this.csv.writeRecord(output_headers);
       
        ResultSet result = database.studentReport2(idgroup,dfecha1, dfecha2);
       
        while (result.next()) {
           
            String nombre= result.getString("name");
            String apellido1= result.getString("surname1");
            String apellido2= result.getString("surname2");
            String materia= result.getString("subject");
            int miss= result.getInt("misscount");
            int excused= result.getInt("excusedcount");
            int delay= result.getInt("delaycount");
            int expulsion= result.getInt("expulsioncount");
           
            String record[] = {apellido1,apellido2,nombre,materia,""+miss+"",""+excused+"",""+delay+"",""+expulsion+""};
            csv.writeRecord(record);
        }
        csv.close();
       
        return ByteArrayOutputStreamCSV;
    }
   
   
    public ByteArrayOutputStream createReport3(int idgroup, Date dfecha1) throws SQLException, IOException, ParseException{

	    	
        ByteArrayOutputStream ByteArrayOutputStreamCSV= new ByteArrayOutputStream();
        
        this.csv = new CsvWriter(ByteArrayOutputStreamCSV, ',', Charset.forName("UTF-8"));
        this.csv.setTextQualifier('\"');
        this.csv.setForceQualifier(true);

        
		
		Calendar calendar = Calendar.getInstance();
	    calendar.setTime(dfecha1);
	    Vector colecciones[] = new Vector[5];
	    //Coger classblocks de cada dia
	    Date fechas[] = new Date[5];
	    for(int k=0;k<5;k++){
	    	fechas[k]=dfecha1;
	        System.out.println(calendar.getTime().toString());
	    	colecciones[k]=(Vector<Classblock>) database.getClassblocksOnDate(idgroup, dfecha1);
	    	calendar.roll(calendar.DATE, true);
	    	dfecha1 = calendar.getTime();
	    }
	    System.out.println("Array de vectores llenos");
	    int size=0;
	    for(int i=0; i<colecciones.length;i++){
	    	size=size+colecciones[i].size();
	    }
	    System.out.println("Tamaño tabla calculado");
	    PdfPTable table = new PdfPTable(size+1);
        
        String output_headers[] = new String[size+1];
        output_headers[0]="Alumno";
	    
        int count=1;
		for(int i=0;i<colecciones.length;i++){
			for(int j=0;j<colecciones[i].size();j++){
				Classblock cb=(Classblock) colecciones[i].elementAt(j);
				output_headers[count]=cb.getStart()+"/"+cb.getEnd();
				count++;
				//PdfPCell aux=new PdfPCell(new Paragraph(new Chunk(df2.format(cb.getStart())+" "+df2.format(cb.getEnd()),new Font(Font.FontFamily.HELVETICA,10))));
			}
		}
		
        this.csv.writeRecord(output_headers);
        
        String fila[]=new String[output_headers.length];
        
        Vector<Student> students=(Vector<Student>) database.getStudentsCollection(idgroup);
		System.out.println("Vector de estudiantes lleno "+students.size());
		//Comprobar faltas y añadir a la tabla
		for(int i=0;i<students.size();i++){
			Student s=students.elementAt(i);
			fila[0]=s.getSurname1()+" "+s.getSurname2()+" ,"+s.getName();
			System.out.println("Estudiante: "+i);
			for(int k=0;k<colecciones.length;k++){
				int count2=1;
				for(int j=0;j<colecciones[k].size();j++){
					Date date=fechas[k];
					Classblock cb=(Classblock) colecciones[k].elementAt(j);
					String text="";
					if(database.existsMiss(1, date, cb.getID(), s.getID())) {
						text=text + " F";
					}	
					if(database.existsMiss(2, date, cb.getID(), s.getID())) {
						text=text + " R";
					}
					if(database.existsMiss(3, date, cb.getID(), s.getID())) {
						text=text + " J";
					}
					if(database.existsMiss(4, date, cb.getID(), s.getID())) {
						text=text + " E";
					}
					fila[count2]=text;
					count2++;
				}
				
		        this.csv.writeRecord(fila);
			}
		}
        
        
        csv.close();
        
        return ByteArrayOutputStreamCSV;
    	
    }
} 