/*
    AttendanceManager
    Copyright (C) 2010  Victor Carceler

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>
 */

package org.iespuigcastellar.attendancemanager.core;

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.net.URL;
import java.net.URLConnection;
import java.util.ResourceBundle;

/**
 * Esta clase permite enviar SMSs. Utiliza una interfaz web para conectar con la API
 * de clickatell.com y enviar un texto al teléfono proporcionado por el usuario. Los
 * datos de conexión están en el fichero properties.
 *
 * @author aalejandro Alex Amigo aalejando@iespuigcastellar.xeill.net
 *
 */
public class SMS {
   
    private ResourceBundle rb = ResourceBundle.getBundle("AttendanceManager");
  
    /**
     * Envia un SMS al teléfono indicado. Este método permite enviar un SMS mediante
	 * clickatell.com dado un número de teléfono y un mensaje para el SMS.
     *
     * @param    text    Una <code>String</code> con el texto a enviar.
     * @param    mobileNumber	Una <code>String</code> con el número de teléfono al
     * que enviar el sms. Se permiten varios números de teléfonos, separados con comas.
     * 
     * @return cierto si se ha enviado el SMS
     */
    public boolean sendSMS(String mobileNumber, String text){
    	       
    	try{
    		URL url = new URL(rb.getString("sms_url") + "user=" + rb.getString("sms_user") + "&password=" + rb.getString("sms_password") +  "&api_id=" + rb.getString("sms_api_id")
    				+ "&to=" + rb.getString("sms_code") + mobileNumber + "&text=" + getHexString(text.getBytes("UTF-16")) + "&unicode=1" + "&concat=" + rb.getString("sms_max_number")
    				+ "&from=" + rb.getString("sms_from"));
    		URLConnection con = url.openConnection();
    		
            BufferedReader in = new BufferedReader(new InputStreamReader(con.getInputStream()));
            String linea;
            while ((linea = in.readLine()) != null) {
            	if(!linea.contains("ID")){
            		Logger.log("SMS.sendSMS(" + mobileNumber + "," + text + ")\n" + linea);
            		return false;
            	}
            }
    		
    		return true;
    		
    	}catch(Exception e){
    		Logger.log("SMS.sendSMS(" + mobileNumber + "," + text + ")");
    		System.err.println(e);
    		return false;
    	}
    }
    
    /**
     * Transforma un texto en su valor en hexadecimal según UTF-16. Dado una <code>Array</code>
     * de <code>Bytes</code> devuelve el mismo mensaje pero con su valor en hexadecimal.
     * 
     * @param	b	Una <code>Array</code> de <code>Bytes</code> con el texto a transformar.
     * @return	Una <code>String</code> con el valor de la cadena en hexadecimal.
     * @throws Exception
     */
    public static String getHexString(byte[] b) throws Exception {
    	  String aux = "";
    	  String result = "";
    	  for (int i=0; i < b.length; i++) {
    	    aux +=
    	          Integer.toString( ( b[i] & 0xff ) + 0x100, 16).substring( 1 );
    	  }
    	  result = aux.substring(4, aux.length());
    	  
    	  return result;
    }
}