/*
    AttendanceManager
    Copyright (C) 2010  Victor Carceler

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>
 */

package org.iespuigcastellar.attendancemanager.core;

public class Subject implements Entity, HasGetName {
	private int id;
	private int idteacher;
	private String name = null;
	private String comment;
	
	/**
	 * Constructor de los nuevos Subject. Al añadir un Subject desde la aplicación,
	 * se usará este constructor.
	 * @param id
	 * @param idteacher
	 * @param name
	 * @param comment
	 */
	public Subject(int id, int idteacher, String name, String comment) {
		this.id = id;
		this.idteacher = idteacher;
		if (name.equals("")) {
			Logger.log("Subject.Subject(" + id + " ," + idteacher + " ," + name + " ," + comment + ")");
			Logger.log("No se puede crear una asignatura con el nombre '' !!!!");
		} else {
			setName(name);
		}
		if(comment.equals("")) setComment(null); else setComment(comment);
	}
	
	/**
	 * Constructor de los Subject importados. Al importar un Subject desde un csv,
	 * se usará este constructor.
	 * @param name
	 * @param comment
	 */
	public Subject(String name, String comment) {
		this(0, 0, name, comment);
	}
	
	/**
	 * Método estático que devuelve un array con las cabeceras del csv
	 * @return devuelve un array con las cadenas de las cabeceras que deberían tener los csv de Subject
	 */
	public static String[] getBasicProperties() {
		String columns[] =  {"name", "comment"};
		return columns;
	}
	public Subject(int id, String name, String comment) {
		this.id = id;
		setName(name);
		setComment(comment);
	}
	public int getID() {
		return id;
	}
	public void setID(int id) {
		this.id = id;
	}
	public int getIDTeacher() {
		return idteacher;
	}
	public void setIDTeacher(int idteacher) {
		this.idteacher = idteacher;
	}
	public String getName() {
		return name;
	}
	public void setName(String name) {
		this.name = Constants.shortString(name);
	}
	public String getComment() {
		return comment;
	}
	public void setComment(String comment) {
		this.comment = Constants.longString(comment);
	}

	@Override
	public String toString() {
		return "Subject [comment=" + comment + ", id=" + id + ", idteacher="
				+ idteacher + ", name=" + name + "]";
	}
	
	/**
	 * Se añade él mismo a la base de datos
	 * @return el id del Subject insertado
	 */
	public int add() {
		return storage.addSubject(this);
	}
	
	/**
	 * Se elimina él mismo de la base de datos
	 */
	public void delete() {
		 storage.deleteSubject(this.id);
	}
	
	/**
	 * Se modifica él mismo en la base de datos
	 * @return el código de retorno
	 */
	public int update() {
		return storage.updateSubject(this);
	}	
}
