package org.iespuigcastellar.attendancemanager.screenlayouts;

import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.net.MalformedURLException;
import java.sql.SQLException;
import java.text.DateFormat;
import java.text.ParseException;
import java.util.Collection;
import java.util.Date;
import java.util.Iterator;
import java.util.Locale;
import java.util.ResourceBundle;


import org.iespuigcastellar.attendancemanager.AttendancemanagerApplication;
import org.iespuigcastellar.attendancemanager.core.CreatePDF;
import org.iespuigcastellar.attendancemanager.core.DownloadResource;
import org.iespuigcastellar.attendancemanager.core.Group;
import org.iespuigcastellar.attendancemanager.core.Mailer;
import org.iespuigcastellar.attendancemanager.core.Term;

import com.itextpdf.text.DocumentException;
import com.vaadin.data.Item;
import com.vaadin.data.Property;
import com.vaadin.data.Property.ValueChangeEvent;
import com.vaadin.data.util.IndexedContainer;
import com.vaadin.terminal.ThemeResource;
import com.vaadin.ui.Alignment;
import com.vaadin.ui.Button;
import com.vaadin.ui.CheckBox;
import com.vaadin.ui.ComboBox;
import com.vaadin.ui.HorizontalLayout;
import com.vaadin.ui.Panel;
import com.vaadin.ui.PopupDateField;
import com.vaadin.ui.TextField;
import com.vaadin.ui.VerticalLayout;
import com.vaadin.ui.Button.ClickEvent;
import com.vaadin.ui.Window.Notification;

public class ReportsLayout extends VerticalLayout {
	private AttendancemanagerApplication app;
	private ResourceBundle rb;
	
	private PopupDateField startDate;
	private PopupDateField endDate;
	
	private ComboBox reportComboBox;
	private ComboBox groupsComboBox;
	private ComboBox termsComboBox;
	
	private CheckBox bydateCheckBox;
	private CheckBox sendmailCheckBox;
	private CheckBox sendmailtoothersCheckBox;
	private CheckBox openordownloadCheckBox;
	private TextField othersTextField;
	private Button executeButton;
	private Button addButton;
	
	private DownloadResource dr;
	private CreatePDF pdf;
	private Mailer mailer;
	
	private void initLayout(){
		//Dates
		//POPUPS DATE
		startDate= new PopupDateField (app.locale.getString("REPORTSLAYOUT_STARTDATE_CAPTION"));
		startDate.setValue(new java.util.Date());
		startDate.setResolution(PopupDateField.RESOLUTION_DAY);
		startDate.setImmediate(true);
		startDate.setEnabled(false);

		endDate= new PopupDateField (app.locale.getString("REPORTSLAYOUT_ENDDATE_CAPTION"));
		endDate.setValue(new java.util.Date());
		endDate.setResolution(PopupDateField.RESOLUTION_DAY);
		endDate.setImmediate(true);
		endDate.setEnabled(false);
		
		//checkboxes
		bydateCheckBox= new CheckBox(app.locale.getString("REPORTSLAYOUT_BYDATECHECKBOX_CAPTION"));
		bydateCheckBox.setImmediate(true);
		
		sendmailtoothersCheckBox= new CheckBox(app.locale.getString("REPORTSLAYOUT_SENDMAILTOOTHERSCHECKBOX_CAPTION"));
		sendmailtoothersCheckBox.setImmediate(true);
		
		sendmailCheckBox=new CheckBox(app.locale.getString("REPORTSLAYOUT_SENDMAILCHECKBOX_CAPTION"));
		sendmailCheckBox.setImmediate(true);
		
		openordownloadCheckBox=new CheckBox(app.locale.getString("REPORTSLAYOUT_OPENORDOWNLOADCHECKBOX_CAPTION"));
		openordownloadCheckBox.setImmediate(true);
		
		
		//TEXTFIELDS
		othersTextField= new TextField("");
		othersTextField.setEnabled(false);
		
		//BUTTONS
		executeButton= new Button(app.locale.getString("REPORTSLAYOUT_EXECUTEBUTTON_CAPTION"));
		executeButton.setIcon(new ThemeResource("img/lightning.png"));
		addButton= new Button(app.locale.getString("REPORTSLAYOUT_ADDBUTTON_CAPTION"));

		//Combos 
		groupsComboBox= new ComboBox(app.locale.getString("REPORTSLAYOUT_GROUPCOMBOBOX_CAPTION"));
		groupsComboBox.setNullSelectionAllowed(false);
		groupsComboBox.setImmediate(true);
		groupsComboBox.setRequired(false);
		groupsComboBox.setFilteringMode(ComboBox.FILTERINGMODE_STARTSWITH);
		groupsComboBox.setReadThrough(true);	
		groupsComboBox.setWidth("100%");
		
		reportComboBox=new ComboBox(app.locale.getString("REPORTSLAYOUT_REPORTCOMBOBOX_CAPTION"));
		reportComboBox.setNullSelectionAllowed(false);
		reportComboBox.setImmediate(true);
		reportComboBox.setRequired(false);
		reportComboBox.setFilteringMode(ComboBox.FILTERINGMODE_STARTSWITH);
		reportComboBox.setReadThrough(true);	
		reportComboBox.setWidth("100%");
		
		termsComboBox=new ComboBox(app.locale.getString("REPORTSLAYOUT_TERMCOMBOBOX_CAPTION"));
		termsComboBox.setNullSelectionAllowed(false);
		termsComboBox.setImmediate(true);
		termsComboBox.setRequired(false);
		termsComboBox.setFilteringMode(ComboBox.FILTERINGMODE_STARTSWITH);
		termsComboBox.setReadThrough(true);	
		termsComboBox.setWidth("100%");
		
		
		//Layouts
		VerticalLayout datesLayout = new VerticalLayout();
		
		datesLayout.addComponent(startDate);
		datesLayout.addComponent(endDate);
		
		VerticalLayout termsdatesLayout = new VerticalLayout();
		termsdatesLayout.setSpacing(true);
		termsdatesLayout.addComponent(termsComboBox);
		termsdatesLayout.addComponent(datesLayout);
		
		
		HorizontalLayout combosLayout= new HorizontalLayout();
		combosLayout.setSpacing(true);
		combosLayout.addComponent(reportComboBox);
		combosLayout.addComponent(groupsComboBox);
		combosLayout.addComponent(termsdatesLayout);
		combosLayout.addComponent(bydateCheckBox);
//		combosLayout.addComponent(datesLayout);
//		combosLayout.addComponent(startDate);
//		combosLayout.addComponent(endDate);
		combosLayout.setComponentAlignment(bydateCheckBox, Alignment.MIDDLE_CENTER);
//		combosLayout.setComponentAlignment(startDate, Alignment.BOTTOM_CENTER);
//		combosLayout.setComponentAlignment(endDate, Alignment.BOTTOM_CENTER);
				
		HorizontalLayout mailersLayout = new HorizontalLayout();
		mailersLayout.setSpacing(true);
		mailersLayout.addComponent(sendmailtoothersCheckBox);
		mailersLayout.addComponent(othersTextField);
		mailersLayout.addComponent(executeButton);
		mailersLayout.setComponentAlignment(sendmailtoothersCheckBox, Alignment.MIDDLE_LEFT);
		
		Panel panelcontents = new Panel(app.locale.getString("REPORTSLAYOUT_PANELCONTENTS_CAPTION"));
		panelcontents.setSizeFull();
		
		panelcontents.addComponent(combosLayout);
		panelcontents.addComponent(openordownloadCheckBox);
		panelcontents.addComponent(sendmailCheckBox);
		panelcontents.addComponent(mailersLayout);
		
		addComponent(panelcontents);
		VerticalLayout vl=(VerticalLayout)panelcontents.getContent();
		vl.setSpacing(true);
		setSizeFull();
		
	}
	public void attach(){
		app=(AttendancemanagerApplication)getApplication();

		pdf= new CreatePDF();
		rb= ResourceBundle.getBundle("AttendanceManager");
		mailer= new Mailer();
		initLayout();
		loadGroups();
		loadTerms();
		loadReports();
		addButton.addListener(new Button.ClickListener() {
			
			@Override
			public void buttonClick(ClickEvent event) {
				// TODO Auto-generated method stub
				TextField newContact= new TextField();
			
				
				
			}
		});
		executeButton.addListener(new Button.ClickListener() {
			
			@Override
			public void buttonClick(ClickEvent event) {
				// TODO Auto-generated method stub
				DateFormat df = DateFormat.getDateInstance(DateFormat.SHORT, new Locale(rb.getString("locale")));

				ByteArrayOutputStream byteArrayOutputStreamPDF;
				boolean sendMail = false;
				String other=null;
				String emailMe=null;
				switch (getSelectedReport()){
				case 0:// Faltas por grupo
					try {
						byteArrayOutputStreamPDF= pdf.createReport1(getSelectedGroup(), getSelectedStartDate(), getSelectedEndDate());
						if(isForOpenReport()){
							System.out.println("Abriendo documento...");
							ByteArrayInputStream bi= new ByteArrayInputStream(byteArrayOutputStreamPDF.toByteArray());

							dr = new DownloadResource(bi, getApplication(), "Informe_"+df.format(new Date())+".pdf", "application/pdf");
							getWindow().open(dr);
						}
						if(isForSendToOthersReport()){
							sendMail = true;
							other = othersTextField.getValue().toString();
						}
						if (isForSendMeReport()){
							sendMail = true;
							emailMe = app.user.getEmail();
						}
						if (sendMail){
							if (emailMe != null && other != null){ 
								String[] to = { other , emailMe };
								mailer.setTo(to);
							}
							else if(emailMe != null){
								String[] to = { emailMe };
								mailer.setTo(to);
							}
							else if(other != null){
								String[] to = { other };
								mailer.setTo(to);
							}
														
							mailer.setSubject("Informe por grupos del " + getSelectedStartDate() + " a " + getSelectedEndDate());
							mailer.setBodyText("Informe del grupo " + getSelectedGroup() + " el día " + getSelectedStartDate() + " al día " + getSelectedEndDate());
							mailer.addMailAttachment(byteArrayOutputStreamPDF.toByteArray(), "application/pdf", "Informe_"+df.format(new Date())+".pdf");
							mailer.sendMail();
/* CONFLICTO VERSIÓN VIEJA !!!
							dr = new DownloadResource(bi, getApplication(), "Informe_por_grupos.pdf", "application/pdf");
							getWindow().open(dr);
						}
						if(isForSendToOthersReport() || isForSendMeReport()){
							sendMail(byteArrayOutputStreamPDF,app.locale.getString("REPORTSLAYOUT_LOADREPORTS_REPORT1"));
*/

						}
						
					} catch (Exception e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					}
					break;
				case 1 :
					try {
						byteArrayOutputStreamPDF= pdf.createReport2(getSelectedGroup(), getSelectedStartDate(), getSelectedEndDate());
						if(isForOpenReport()){
							System.out.println("Abriendo documento...");
							ByteArrayInputStream bi= new ByteArrayInputStream(byteArrayOutputStreamPDF.toByteArray());

							dr = new DownloadResource(bi, getApplication(), "Informe_"+df.format(new Date())+".pdf", "application/pdf");
							getWindow().open(dr);
						}
						if(isForSendToOthersReport()){
							sendMail = true;
							other = othersTextField.getValue().toString();
						}
						if (isForSendMeReport()){
							sendMail = true;
							emailMe = app.user.getEmail();
						}
						if (sendMail){
							if (emailMe != null && other != null){ 
								String[] to = { other , emailMe };
								mailer.setTo(to);
							}
							else if(emailMe != null){
								String[] to = { emailMe };
								mailer.setTo(to);
							}
							else if(other != null){
								String[] to = { other };
								mailer.setTo(to);
							}
														
							mailer.setSubject("Informe por asignatura del " + getSelectedStartDate() + " a " + getSelectedEndDate());
							mailer.setBodyText("Informe del grupo " + getSelectedGroup() + " el día " + getSelectedStartDate() + " al día " + getSelectedEndDate());
							mailer.addMailAttachment(byteArrayOutputStreamPDF.toByteArray(), "application/pdf", "Informe_"+df.format(new Date())+".pdf");
							mailer.sendMail();

/*
							dr = new DownloadResource(bi, getApplication(), "Informe_por_asignaturas.pdf", "application/pdf");
							getWindow().open(dr);
						}
						if(isForSendToOthersReport() || isForSendMeReport()){
							sendMail(byteArrayOutputStreamPDF,app.locale.getString("REPORTSLAYOUT_LOADREPORTS_REPORT2"));
*/

						}
					} catch (Exception e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					}
					break;
				case 2 :
					try {
						byteArrayOutputStreamPDF= pdf.createReport3(getSelectedGroup(), getSelectedStartDate());
						if(isForOpenReport()){
							System.out.println("Abriendo documento...");
							ByteArrayInputStream bi= new ByteArrayInputStream(byteArrayOutputStreamPDF.toByteArray());

							dr = new DownloadResource(bi, getApplication(), "Informe_"+df.format(new Date())+".pdf", "application/pdf");
							getWindow().open(dr);
						}
						if(isForSendToOthersReport()){
							sendMail = true;
							other = othersTextField.getValue().toString();
						}
						if (isForSendMeReport()){
							sendMail = true;
							emailMe = app.user.getEmail();
						}
						if (sendMail){
							if (emailMe != null && other != null){ 
								String[] to = { other , emailMe };
								mailer.setTo(to);
							}
							else if(emailMe != null){
								String[] to = { emailMe };
								mailer.setTo(to);
							}
							else if(other != null){
								String[] to = { other };
								mailer.setTo(to);
							}
														
							mailer.setSubject("Informe semanal del " + getSelectedStartDate() + " a " + getSelectedEndDate());
							mailer.setBodyText("Informe semanal del grupo " + getSelectedGroup() + " el día " + getSelectedStartDate() + " al día " + getSelectedEndDate());
							mailer.addMailAttachment(byteArrayOutputStreamPDF.toByteArray(), "application/pdf_", "Informe"+df.format(new Date())+".pdf");
							mailer.sendMail();

/*
							dr = new DownloadResource(bi, getApplication(), "Informe_semanal.pdf", "application/pdf");
							getWindow().open(dr);
						}
						if(isForSendToOthersReport() || isForSendMeReport()){
							sendMail(byteArrayOutputStreamPDF,app.locale.getString("REPORTSLAYOUT_LOADREPORTS_REPORT3"));
*/

						}
					} catch (Exception e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					}
					break;
					
				}
				
			}
		});
				  /*getWindow().showNotification(
						  app.locale.getString("REPORTSLAYOUT_EXECUTEWINDOW_REPORT") + " " + getSelectedReport()
						  + app.locale.getString("REPORTSLAYOUT_EXECUTEWINDOW_GROUP") + " " + getSelectedGroup()
						  + app.locale.getString("REPORTSLAYOUT_EXECUTEWINDOW_TERM") + " " + getSelectedTerm()
						  + app.locale.getString("REPORTSLAYOUT_EXECUTEWINDOW_STARTDATE") + " " + getSelectedStartDate()
						  + app.locale.getString("REPORTSLAYOUT_EXECUTEWINDOW_ENDDATE") + " " + getSelectedEndDate()
						  + app.locale.getString("REPORTSLAYOUT_EXECUTEWINDOW_DOCUMENT") + " " + isForOpenReport()
						  + app.locale.getString("REPORTSLAYOUT_EXECUTEWINDOW_SENDMAIL") + " " + isForSendToOthersReport()
						  + app.locale.getString("REPORTSLAYOUT_EXECUTEWINDOW_SENDMAILTOOTHERS") + " " + isForSendToOthersReport());*/
				  
		bydateCheckBox.addListener(new Property.ValueChangeListener() {
			
			@Override
			public void valueChange(ValueChangeEvent event) {
				// TODO Auto-generated method stub
					termsComboBox.setEnabled(! bydateCheckBox.booleanValue());
					startDate.setEnabled(bydateCheckBox.booleanValue());
					endDate.setEnabled(bydateCheckBox.booleanValue());
				}
				
			
		});
		sendmailtoothersCheckBox.addListener(new Property.ValueChangeListener() {
			
			@Override
			public void valueChange(ValueChangeEvent event) {
				// TODO Auto-generated method stub
				othersTextField.setEnabled(sendmailtoothersCheckBox.booleanValue());
				}
				
			
		});
		
		reportComboBox.addListener(new Property.ValueChangeListener(){

			@Override
			public void valueChange(ValueChangeEvent event) {
				// TODO Auto-generated method stub
				if((Integer)reportComboBox.getValue()==2){ 
					startDate.setEnabled(true);
					endDate.setEnabled(true);
					termsComboBox.setEnabled(false);
					}
				else { 
					termsComboBox.setEnabled(true);
					startDate.setEnabled(false);
					endDate.setEnabled(false);
				}
			}

			
		});
		
	}
	private String getEmailOthers(){
		if (isForSendToOthersReport()){
			return (String) othersTextField.getValue();
		}else{
			return null;
		}
	}
	private Boolean isForSendToOthersReport(){
		return sendmailtoothersCheckBox.booleanValue();
	}
	private Boolean isForSendMeReport(){	
		return sendmailCheckBox.booleanValue();
	}
	private Boolean isForOpenReport(){
		return openordownloadCheckBox.booleanValue();
	}
	private Integer getSelectedGroup(){
		return (Integer) groupsComboBox.getValue();
	}
	private java.util.Date getSelectedStartDate(){
		if (bydateCheckBox.booleanValue()){
			return (Date) startDate.getValue();
		}else{
			return (Date) termsComboBox.getContainerDataSource().getContainerProperty(termsComboBox.getValue(), "start").getValue();
		}
	}
	private java.util.Date getSelectedEndDate(){
		if (bydateCheckBox.booleanValue()){
			return (Date) endDate.getValue();
		}else{termsComboBox.setEnabled(! bydateCheckBox.booleanValue());
			return (Date) termsComboBox.getContainerDataSource().getContainerProperty(termsComboBox.getValue(), "end").getValue();
		}
	}
	private Integer getSelectedTerm(){
		if (! bydateCheckBox.booleanValue()){
			return (Integer) termsComboBox.getValue();
		}else{
			return null;
		}
	}
	private Integer getSelectedReport(){
		return (Integer) reportComboBox.getValue();
	}
	private void loadReports(){
		IndexedContainer ic=new IndexedContainer();
		ic.addContainerProperty("namereport", String.class, null);
	
	
		Item itemTerm=ic.addItem(0);
		itemTerm.getItemProperty("namereport").setValue(app.locale.getString("REPORTSLAYOUT_LOADREPORTS_REPORT1"));
		itemTerm=ic.addItem(1);
		itemTerm.getItemProperty("namereport").setValue(app.locale.getString("REPORTSLAYOUT_LOADREPORTS_REPORT2"));
		itemTerm=ic.addItem(2);
		itemTerm.getItemProperty("namereport").setValue(app.locale.getString("REPORTSLAYOUT_LOADREPORTS_REPORT3"));
		
		reportComboBox.setContainerDataSource(ic);
		reportComboBox.setItemCaptionPropertyId("namereport");
		reportComboBox.setValue(0);
	}
//	private void loadGroups(){
//		//Falta pasar el teacher user en allGroups pasado 1 provisionalmente 
//
//		Collection collection = app.storage.allGroups(app.user);
//
//		IndexedContainer ic=new IndexedContainer();
//		ic.addContainerProperty("namegroup", String.class, null);
//		Iterator<Group> it =collection.iterator();
//		Item itemGroup=ic.addItem(0);
//		itemGroup.getItemProperty("namegroup").setValue(app.locale.getString("REPORTSLAYOUT_LOADTERMS_ALLGROUPS"));
//		while (it.hasNext()){
//			Group group=it.next();
//			itemGroup=ic.addItem(group.getId());
//			itemGroup.getItemProperty("namegroup").setValue(group.getName());
//		}
//		groupsComboBox.setContainerDataSource(ic);
//		groupsComboBox.setItemCaptionPropertyId("namegroup");
//		groupsComboBox.setValue(0);
//	}
	private void loadGroups(){
        //Falta pasar el teacher user en allGroups pasado 1 provisionalmente

        Collection collection = app.storage.allGroups(app.user);

        IndexedContainer ic=new IndexedContainer();
        ic.addContainerProperty("namegroup", String.class, null);
        Iterator<Group> it =collection.iterator();
        Integer firstItem=null;
        Item itemGroup=null;//ic.addItem(0);
        //itemGroup.getItemProperty("namegroup").setValue(app.locale.getString("REPORTSLAYOUT_LOADTERMS_ALLGROUPS"));
        while (it.hasNext()){
            Group group=it.next();
            if (firstItem==null){firstItem=group.getId();}
            itemGroup=ic.addItem(group.getId());
            itemGroup.getItemProperty("namegroup").setValue(group.getName());
        }
        groupsComboBox.setContainerDataSource(ic);
        groupsComboBox.setItemCaptionPropertyId("namegroup");
        if (firstItem!=null){groupsComboBox.setValue(firstItem);};
    }
//	private void loadTerms(){
//
//		Collection collection = app.storage.allTerms();
//		IndexedContainer ic=new IndexedContainer();
//		ic.addContainerProperty("term", String.class, null);
//		ic.addContainerProperty("start", java.util.Date.class, null);
//		ic.addContainerProperty("end", java.util.Date.class, null);
//		Iterator<Term> it =collection.iterator();
//		Item itemTerm=ic.addItem(0);
//		itemTerm.getItemProperty("term").setValue(app.locale.getString("REPORTSLAYOUT_LOADTERMS_ALLTERMS"));
//		while (it.hasNext()){
//			Term term=it.next();
//			itemTerm=ic.addItem(term.getIdTerm());
//			itemTerm.getItemProperty("term").setValue(term.getName());
//			itemTerm.getItemProperty("start").setValue(term.getStart());
//			itemTerm.getItemProperty("end").setValue(term.getEnd());
//		}
//		termsComboBox.setContainerDataSource(ic);
//		termsComboBox.setItemCaptionPropertyId("term");
//		termsComboBox.setValue(0);	
//	}
	private void loadTerms(){
        Collection collection = app.storage.allTerms();
        IndexedContainer ic=new IndexedContainer();
        ic.addContainerProperty("term", String.class, null);
        ic.addContainerProperty("start", java.util.Date.class, null);
        ic.addContainerProperty("end", java.util.Date.class, null);
        Iterator<Term> it =collection.iterator();
        Integer firstItem=null;
        Item itemTerm=null;//ic.addItem(0);
        //itemTerm.getItemProperty("term").setValue(app.locale.getString("REPORTSLAYOUT_LOADTERMS_ALLTERMS"));
        while (it.hasNext()){
            Term term=it.next();
            if (firstItem==null){firstItem=term.getIdTerm();}
            itemTerm=ic.addItem(term.getIdTerm());
            itemTerm.getItemProperty("term").setValue(term.getName());
            itemTerm.getItemProperty("start").setValue(term.getStart());
            itemTerm.getItemProperty("end").setValue(term.getEnd());
        }
        termsComboBox.setContainerDataSource(ic);
        termsComboBox.setItemCaptionPropertyId("term");
        if (firstItem!=null){termsComboBox.setValue(firstItem);};   
    	}

	private void sendMail(ByteArrayOutputStream byteArrayOutputStreamPDF , String report){
		boolean sendMail = false;
		String other="";
		String emailMe="";
		if(isForSendToOthersReport()){
			sendMail = true;
			other = othersTextField.getValue().toString();
		}
		if (isForSendMeReport()){
			sendMail = true;
			emailMe = app.user.getEmail();
		}
		if (sendMail){
			String unvalidAddress ="";
			boolean sendIt = false;
			if (mailer.isValidEmailAddress(emailMe) && mailer.isValidEmailAddress(other)){ 
				String[] to = { other , emailMe };
				mailer.setTo(to);
				sendIt = true;
			}
			else if (!mailer.isValidEmailAddress(emailMe) && !mailer.isValidEmailAddress(other)){
				unvalidAddress = other +" "+ emailMe;
			}
			else if(mailer.isValidEmailAddress(emailMe) && !mailer.isValidEmailAddress(other)){
				String[] to = { emailMe };
				mailer.setTo(to);
				unvalidAddress = other;
				sendIt = true;
			}
			else if(!mailer.isValidEmailAddress(emailMe) && mailer.isValidEmailAddress(other)){
				String[] to = { other };
				mailer.setTo(to);
				unvalidAddress = emailMe;
				sendIt = true;
			}
			
			if (sendIt){
				mailer.setSubject(report);
				mailer.setBodyText("Generación de informes de Gestión de faltas de asistencia\n\t- "+report+" adjunto");
				mailer.addMailAttachment(byteArrayOutputStreamPDF.toByteArray(), "application/pdf", report+".pdf");
				mailer.sendMail();
				if (unvalidAddress == ""){
					getParent().getWindow().showNotification(
							app.locale.getString("REPORTSLAYOUT_SUCCESSMAIL_MESSAGE") + mailer.getToElements(),
							Notification.TYPE_TRAY_NOTIFICATION);
				}
				else{
				getParent().getWindow().showNotification(
						app.locale.getString("REPORTSLAYOUT_SUCCESSMAIL_MESSAGE") + mailer.getToElements(),
						app.locale.getString("REPORTSLAYOUT_UNSUCCESSMAIL_MESSAGE") + unvalidAddress ,
						Notification.TYPE_TRAY_NOTIFICATION);
				}
			}
			else{
				getParent().getWindow().showNotification(
						app.locale.getString("REPORTSLAYOUT_FAILSENDMAIL_MESSAGE"), unvalidAddress , Notification.TYPE_ERROR_MESSAGE);
			}
		}
	}

}
