/*
    AttendanceManager
    Copyright (C) 2010  Victor Carceler

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>
 */

/*
 *	GroupManagerLayout designed by Xavier Junyent <xavierjunyent@yahoo.es>
 *
 *	GroupManagerLayout allows group administration.
 */

package org.iespuigcastellar.attendancemanager.screenlayouts;

import java.sql.SQLException;
import java.util.Arrays;
import java.util.Collection;
import java.util.Date;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;



import org.iespuigcastellar.attendancemanager.AttendancemanagerApplication;
import org.iespuigcastellar.attendancemanager.core.CSVUploader;
import org.iespuigcastellar.attendancemanager.core.Constants;
import org.iespuigcastellar.attendancemanager.core.DatabaseStorage;
import org.iespuigcastellar.attendancemanager.core.Group;
import org.iespuigcastellar.attendancemanager.core.Student;

import com.vaadin.data.Item;
import com.vaadin.data.Property;
import com.vaadin.data.Property.ValueChangeEvent;
import com.vaadin.data.util.BeanItem;
import com.vaadin.data.util.QueryContainer;
import com.vaadin.data.validator.EmailValidator;
import com.vaadin.data.validator.RegexpValidator;
import com.vaadin.data.validator.StringLengthValidator;
import com.vaadin.terminal.ThemeResource;
import com.vaadin.ui.Alignment;
import com.vaadin.ui.Button;
import com.vaadin.ui.CheckBox;
import com.vaadin.ui.Component;
import com.vaadin.ui.CssLayout;
import com.vaadin.ui.Embedded;
import com.vaadin.ui.Form;
import com.vaadin.ui.HorizontalLayout;
import com.vaadin.ui.Label;
import com.vaadin.ui.Panel;
import com.vaadin.ui.SplitPanel;
import com.vaadin.ui.Table;
import com.vaadin.ui.TextField;
import com.vaadin.ui.Upload;
import com.vaadin.ui.VerticalLayout;
import com.vaadin.ui.Window;
import com.vaadin.ui.Button.ClickEvent;
import com.vaadin.ui.Upload.FailedEvent;
import com.vaadin.ui.Upload.SucceededEvent;
import com.vaadin.ui.Window.Notification;


@SuppressWarnings("serial")
public class StudentsManagerLayout extends VerticalLayout {
	
	private AttendancemanagerApplication app;
	
	private Button newButton;
	private Button editButton;
	private Button deleteButton;
	private Button importButton;
	private Button addgroupButton;
	private Button removegroupButton;
	
	private Table studentsTable;
	private Table groupStudentsTable;
	private Table groupNoStudentsTable;
	
	private NewStudentWindow newStudentWindow;
	private EditStudentWindow editStudentWindow;
	private DeleteStudentWindow deleteStudentWindow;
	private ImportStudentWindow importStudentWindow;
	
	private void initLayout() {

		addgroupButton= new Button();
		addgroupButton.setIcon(new ThemeResource("img/24-em-up.png"));
		addgroupButton.setEnabled(false);
		addgroupButton.setWidth("100%");
		addgroupButton.setStyleName("small");
		
		removegroupButton= new Button();
		removegroupButton.setIcon(new ThemeResource("img/24-em-down.png"));
		removegroupButton.setEnabled(false);
		removegroupButton.setWidth("100%");
		removegroupButton.setStyleName("small");
		
		newButton = new Button();
		newButton.setCaption(app.locale.getString("GROUPMANAGERLAYOUT_NEWBUTTON_CAPTION"));
		newButton.setIcon(new ThemeResource("img/user_add.png"));
		newButton.setStyleName("default");
		
	
		editButton = new Button();
		editButton.setCaption(app.locale.getString("GROUPMANAGERLAYOUT_EDITBUTTON_CAPTION"));
		editButton.setIcon(new ThemeResource("img/user_edit.png"));
		editButton.setEnabled(false);
		editButton.setStyleName("default");
		
		deleteButton = new Button();
		deleteButton.setCaption(app.locale.getString("GROUPMANAGERLAYOUT_DELETEBUTTON_CAPTION"));
		deleteButton.setIcon(new ThemeResource("img/user_delete.png"));
		deleteButton.setEnabled(false);
		deleteButton.setStyleName("default");
		
		importButton = new Button();
		importButton.setCaption(app.locale.getString("GROUPMANAGERLAYOUT_IMPORTBUTTON_CAPTION"));
		importButton.setIcon(new ThemeResource("img/user_go.png"));
		importButton.setStyleName("default");
		
		CssLayout toolbarstudents = new CssLayout();
		toolbarstudents.addComponent(newButton);
		toolbarstudents.addComponent(editButton);
		toolbarstudents.addComponent(deleteButton);
		toolbarstudents.addComponent(importButton);
		toolbarstudents.setStyleName("toolbar");
		toolbarstudents.setWidth("100%");

		HorizontalLayout toolbargroupsnostudent = new HorizontalLayout();
		toolbargroupsnostudent.addComponent(addgroupButton);
		toolbargroupsnostudent.setSpacing(true);
		toolbargroupsnostudent.setMargin(false);
		toolbargroupsnostudent.setWidth("100%");
		toolbargroupsnostudent.setExpandRatio(addgroupButton, 1);
		toolbargroupsnostudent.setComponentAlignment(addgroupButton, Alignment.MIDDLE_CENTER);
		
		HorizontalLayout toolbargroupsstudent = new HorizontalLayout();
		toolbargroupsstudent.addComponent(removegroupButton);
		toolbargroupsstudent.setSpacing(true);
		toolbargroupsstudent.setMargin(false);
		toolbargroupsstudent.setWidth("100%");
		toolbargroupsstudent.setExpandRatio(removegroupButton, 1);
		toolbargroupsstudent.setComponentAlignment(removegroupButton, Alignment.MIDDLE_CENTER);
		
		studentsTable = new Table();
		studentsTable.setImmediate(true);
		studentsTable.setColumnReorderingAllowed(true);
		studentsTable.setColumnCollapsingAllowed(true);
		studentsTable.setStyleName("striped");
		studentsTable.setSizeFull();
		
		groupStudentsTable = new Table();
		groupStudentsTable.setImmediate(true);
		groupStudentsTable.setColumnReorderingAllowed(true);
		groupStudentsTable.setColumnCollapsingAllowed(true);
		groupStudentsTable.setStyleName("striped");
		groupStudentsTable.setSizeFull();
		
		groupNoStudentsTable = new Table();
		groupNoStudentsTable.setImmediate(true);
		groupNoStudentsTable.setColumnReorderingAllowed(true);
		groupNoStudentsTable.setColumnCollapsingAllowed(true);
		groupNoStudentsTable.setStyleName("striped");
		groupNoStudentsTable.setSizeFull();

		
		Panel studentsPanel = new Panel();
		studentsPanel.setCaption(app.locale.getString("STUDENTSMANAGERLAYOUT_STUDENTSPANEL_CAPTION"));
		studentsPanel.addComponent(studentsTable);
		studentsPanel.setSizeFull();
		studentsPanel.setStyleName("bubble");
		studentsPanel.setIcon(new ThemeResource("img/user.png"));
		studentsPanel.getContent().setSizeFull();
		
		Panel groupsstudentsPanel = new Panel();
		groupsstudentsPanel.setCaption(app.locale.getString("STUDENTSMANAGERLAYOUT_GROUPSSTUDENTSPANEL_CAPTION"));
		groupsstudentsPanel.addComponent(groupStudentsTable);
		groupsstudentsPanel.addComponent(toolbargroupsstudent);
		groupsstudentsPanel.setSizeFull();
		groupsstudentsPanel.setStyleName("bubble");
		groupsstudentsPanel.setIcon(new ThemeResource("img/group.png"));
		groupsstudentsPanel.getContent().setSizeFull();
		
		Panel groupsNoStudentPanel = new Panel();
		groupsNoStudentPanel.setCaption(app.locale.getString("STUDENTSMANAGERLAYOUT_GROUPSNOSTUDENTSPANEL_CAPTION"));
		groupsNoStudentPanel.addComponent(groupNoStudentsTable);
		groupsNoStudentPanel.setSizeFull();
		groupsNoStudentPanel.setStyleName("bubble");
		groupsNoStudentPanel.setIcon(new ThemeResource("img/group.png"));
		groupsNoStudentPanel.getContent().setSizeFull();
		
		VerticalLayout vl =  (VerticalLayout) groupsNoStudentPanel.getContent();
		vl.setMargin(false);
		vl.setExpandRatio(groupNoStudentsTable, 1.0f);
		vl.setSpacing(false);

		
		vl=(VerticalLayout) groupsstudentsPanel.getContent();
		vl.setMargin(false);
		vl.setExpandRatio(groupStudentsTable, 1.0f);
		vl.setSpacing(false);
	
		
		vl =  (VerticalLayout) studentsPanel.getContent();
		vl.setMargin(false);
		vl.setExpandRatio(studentsTable, 1.0f);
		vl.setSpacing(false);
		
		//We want toolbarstudentsnogroup must be on top
		
		VerticalLayout groupsnostudentLayout = new VerticalLayout();
		groupsnostudentLayout.addComponent(toolbargroupsnostudent);
		groupsnostudentLayout.addComponent(groupsNoStudentPanel);
		groupsnostudentLayout.setSizeFull();
		groupsnostudentLayout.setExpandRatio(groupsNoStudentPanel, 1);
		
		SplitPanel tablesGroupsSplitPanel = new SplitPanel(SplitPanel.ORIENTATION_VERTICAL);
		tablesGroupsSplitPanel.addComponent(groupsstudentsPanel);
		tablesGroupsSplitPanel.addComponent(groupsnostudentLayout);
		tablesGroupsSplitPanel.setSizeFull();

		SplitPanel tablesSplitPanel = new SplitPanel(SplitPanel.ORIENTATION_HORIZONTAL);
		tablesSplitPanel.addComponent(studentsPanel);
		tablesSplitPanel.addComponent(tablesGroupsSplitPanel);
		tablesSplitPanel.setSizeFull();
		
		addComponent(toolbarstudents);
		addComponent(tablesSplitPanel);
		
		setSizeFull();
		setMargin(false, true, false, true);
		
		setExpandRatio(tablesSplitPanel, 1.0f);
		setSpacing(true);
	}
	public void attach() {
		app = (AttendancemanagerApplication)getApplication();
		initLayout();
		studentsTable.addContainerProperty(DatabaseStorage.SURNAME1_PROPERTY, String.class, "");
		studentsTable.addContainerProperty(DatabaseStorage.SURNAME2_PROPERTY, String.class, "");
		studentsTable.addContainerProperty(DatabaseStorage.NAME_PROPERTY, String.class, "");
		studentsTable.addContainerProperty(DatabaseStorage.EMAIL_PROPERTY, String.class, "");
		studentsTable.addContainerProperty(DatabaseStorage.SMSPHONE_PROPERTY, String.class, "");
		studentsTable.setColumnHeaders(new String[] { app.locale.getString("GROUPMANAGERLAYOUT_STUDENTGROUPTABLE_SURNAME1"),app.locale.getString("GROUPMANAGERLAYOUT_STUDENTGROUPTABLE_SURNAME2"),app.locale.getString("GROUPMANAGERLAYOUT_GROUPSTABLE_NAME"),app.locale.getString("GROUPMANAGERLAYOUT_STUDENTGROUPTABLE_EMAIL"),app.locale.getString("GROUPMANAGERLAYOUT_STUDENTGROUPTABLE_SMSPHONE") });
		
		app.storage.refreshTable(app.storage.getStudentsQueryContainer(), studentsTable);
		studentsTable.setSelectable(true);
		studentsTable.setMultiSelect(false);
		studentsTable.setImmediate(true);
		groupStudentsTable.addContainerProperty(DatabaseStorage.NAME_PROPERTY, String.class, "");
		groupStudentsTable.addContainerProperty(DatabaseStorage.COMMENT_PROPERTY, String.class, "");
		groupStudentsTable.setColumnHeaders(new String[] { app.locale.getString("GROUPMANAGERLAYOUT_GROUPSTABLE_NAME"),app.locale.getString("GROUPMANAGERLAYOUT_GROUPSTABLE_COMMENT") });
			//app.storage.refreshTable(app.storage.getStudentsQueryContainer(), studentsGroupTable);
		groupStudentsTable.setSelectable(true);
		groupStudentsTable.setMultiSelect(true);
		groupStudentsTable.setImmediate(true);
		groupStudentsTable.addListener(new Property.ValueChangeListener() {
			
			@Override
			public void valueChange(ValueChangeEvent event) {
				// TODO Auto-generated method stub
				Set<?> value = (Set<?>) event.getProperty().getValue();
				if (null == value || value.size() == 0) {
					removegroupButton.setEnabled(false);

				} else {
					removegroupButton.setEnabled(true);
				}
				
			}
		});
		
		groupNoStudentsTable.addContainerProperty(DatabaseStorage.NAME_PROPERTY, String.class, "");
		groupNoStudentsTable.addContainerProperty(DatabaseStorage.COMMENT_PROPERTY, String.class, "");
		//groupNoStudentsTable.addContainerProperty(DatabaseStorage.STUDENTS_PROPERTY, Integer.class, 0);
		groupNoStudentsTable.setColumnHeaders(new String[] { app.locale.getString("GROUPMANAGERLAYOUT_GROUPSTABLE_NAME"),app.locale.getString("GROUPMANAGERLAYOUT_GROUPSTABLE_COMMENT") });
		//app.storage.refreshTable(app.storage.getStudentsQueryContainer(), studentsNoGroupTable);
		groupNoStudentsTable.setSelectable(true);
		groupNoStudentsTable.setMultiSelect(true);
		groupNoStudentsTable.setImmediate(true);
		groupNoStudentsTable.addListener(new Property.ValueChangeListener() {
			
			@Override
			public void valueChange(ValueChangeEvent event) {
				// TODO Auto-generated method stub
				Set<?> value = (Set<?>) event.getProperty().getValue();
				if (null == value || value.size() == 0) {
					addgroupButton.setEnabled(false);

				} else {
					addgroupButton.setEnabled(true);
				}
				
			}
		});
		studentsTable.addListener(new Property.ValueChangeListener() {
			
			@Override
			public void valueChange(ValueChangeEvent event) {
				Object value = studentsTable.getValue();
				if (value!=null){
					refreshGroups((Integer)value);
					editButton.setEnabled(true);
					deleteButton.setEnabled(true);
				} else {
					groupStudentsTable.removeAllItems();
					groupNoStudentsTable.removeAllItems();
					editButton.setEnabled(false);
					deleteButton.setEnabled(false);
					removegroupButton.setEnabled(false);
					addgroupButton.setEnabled(false);
					
				}
			}
		});

		newStudentWindow = new NewStudentWindow();
		newStudentWindow.setCaption(app.locale.getString("STUDENTSMANAGERLAYOUT_NEWSTUDENTWINDOW_CAPTION"));
		
		editStudentWindow = new EditStudentWindow();
		editStudentWindow.setCaption(app.locale.getString("STUDENTSMANAGERLAYOUT_EDITSTUDENTWINDOW_CAPTION"));
				
		deleteStudentWindow= new DeleteStudentWindow();
		deleteStudentWindow.setCaption(app.locale.getString("STUDENTSMANAGERLAYOUT_DELETESTUDENTWINDOW_CAPTION"));
				
		try {
			importStudentWindow= new ImportStudentWindow();
		} catch (SecurityException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (NoSuchMethodException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		importStudentWindow.setCaption(app.locale.getString("STUDENTSMANAGERLAYOUT_IMPORTSTUDENTWINDOW_CAPTION"));
		
		newButton.addListener(new Button.ClickListener() {
			
			@Override
			public void buttonClick(ClickEvent event) {
				// Open window if not open already
				if(newStudentWindow.getParent() != null) {
					// Window already open
				} else {
					// open window
					getWindow().addWindow(newStudentWindow);
				}
			}
		});
		
		editButton.addListener(new Button.ClickListener() {
			
			@Override
			public void buttonClick(ClickEvent event) {
				// TODO Auto-generated method stub
				Integer idItem=(Integer) studentsTable.getValue();
				Item item=studentsTable.getItem(idItem);
				Integer idStudent=(Integer) studentsTable.getValue();
				String name=(String) item.getItemProperty("name").getValue();
				String surname1=(String) item.getItemProperty("surname1").getValue();
				String surname2=(String) item.getItemProperty("surname2").getValue();
				String email=(String) item.getItemProperty("email").getValue();
				String smsphone=(String) item.getItemProperty("smsphone").getValue();
				

				editStudentWindow.setIdStudent(idStudent);
				editStudentWindow.studentForm.getField("name").setValue(name);
				editStudentWindow.studentForm.getField("surname1").setValue(surname1);
				editStudentWindow.studentForm.getField("surname2").setValue(surname2);
				editStudentWindow.studentForm.getField("email").setValue(email);
				editStudentWindow.studentForm.getField("smsphone").setValue(smsphone);
				getWindow().addWindow(editStudentWindow);
			}
		});
		
		deleteButton.addListener(new Button.ClickListener() {
			
			@Override
			public void buttonClick(ClickEvent event) {
				// TODO Auto-generated method stub
				Integer idItem=(Integer) studentsTable.getValue();
				Item item=studentsTable.getItem(idItem);
				Integer idStudent=(Integer) studentsTable.getValue();
				String name=(String) item.getItemProperty("name").getValue();
				String surname1=(String) item.getItemProperty("surname1").getValue();
				String surname2=(String) item.getItemProperty("surname2").getValue();
				String email=(String) item.getItemProperty("email").getValue();
				String smsphone=(String) item.getItemProperty("smsphone").getValue();;
				deleteStudentWindow.setStudent(new Student(idStudent,name,surname1,surname2,email,smsphone));
				getWindow().addWindow(deleteStudentWindow);
			}
		});
		importButton.addListener(new Button.ClickListener() {
			
			@Override
			public void buttonClick(ClickEvent event) {
				// TODO Auto-generated method stub
				getWindow().addWindow(importStudentWindow);
			}
		});
		
		addgroupButton.addListener(new Button.ClickListener() {
			@Override
			public void buttonClick(ClickEvent event) {
				// TODO Auto-generated method stub
				Set<?> value = (Set<?>) groupNoStudentsTable.getValue();
				if (null != value && value.size() > 0) {
					Iterator it=value.iterator();
					Integer idstudent =(Integer) studentsTable.getValue();
					while(it.hasNext()){
						
						Integer idgroup=(Integer) it.next();
						app.storage.addStudentToGroup(idstudent,idgroup);
					}
					Integer itemid=(Integer) studentsTable.getValue();
					//Integer valuestudents =(Integer) studentsTable.getContainerProperty(itemid,"students").getValue();
					//Integer newvaluestudents=valuestudents + value.size();
					//studentsTable.getContainerProperty(itemid,"students").setValue(newvaluestudents);
					refreshGroups(idstudent);

				}
			}
		});
		
		removegroupButton.addListener(new Button.ClickListener() {
			@Override
			public void buttonClick(ClickEvent event) {
				// TODO Auto-generated method stub;
				Set<?> value = (Set<?>) groupStudentsTable.getValue();
				if (null != value && value.size() > 0) {
					Iterator it=value.iterator();
					Integer idstudent =(Integer) studentsTable.getValue();
					while(it.hasNext()){
						Integer idgroup=(Integer) it.next();
						app.storage.deleteStudentFromGroup(idgroup,idstudent);
					}
				//	Integer itemid=(Integer) studentsTable.getValue();
				//	Integer valuestudents =(Integer) studentsTable.getContainerProperty(itemid,"students").getValue();
					refreshGroups(idstudent);
				}
			}
		});

	}
	public void refreshLayout(){
		refreshStudents();
	}
	public void refreshStudents() {
		Integer id= (Integer) studentsTable.getValue();
		app.storage.refreshTable(app.storage.getStudentsQueryContainer(), studentsTable);
		if (id!=null){studentsTable.setValue(id);}
	}
	public void refreshGroups(Integer idstudent) {
		app.storage.refreshTable(app.storage.getGroupsOfStudentQueryContainer(idstudent), groupStudentsTable);
		app.storage.refreshTable(app.storage.getGroupsHaveNotStudentGroupQueryContainer(idstudent), groupNoStudentsTable);
	}
	/**
	 * Updates selection in students table to show members of group
	 */
	public void selectStudents() {
		
	
	}
	
	/*
	 * Inner classes
	 */
	public class StudentInfo {
		
		
		public String surname1 = "";
		public String surname2 = "";
		public String name = "";
		public String email = "";
		public String smsphone = "";
		
		
		public String getName() {
			return name;
		}
		public void setName(String name) {
			this.name = name;
		}
		public String getSurname1() {
			return surname1;
		}
		public void setSurname1(String surname1) {
			this.surname1 = surname1;
		}		
		public String getSurname2() {
			return surname2;
		}
		public void setSurname2(String surname2) {
			this.surname2 = surname2;
		}
		public String getEmail() {
			return email;
		}
		public void setEmail(String email) {
			this.email = email;
		}
		public String getSmsphone() {
			return smsphone;
		}
		public void setSmsphone(String smsphone) {
			this.smsphone = smsphone;
		}
		
	}
	
	private class NewStudentWindow extends Window {
		
		private final Form studentForm = new Form();
		private final StudentInfo studentInfo = new StudentInfo();
		
		public NewStudentWindow() {
			
			studentForm.setValidationVisible(true);
			
			BeanItem<StudentInfo> groupBean = new BeanItem<StudentInfo>(studentInfo);
			studentForm.setItemDataSource(groupBean,Arrays.asList ( "name","surname1","surname2","email","smsphone"));
			
			TextField tf = (TextField)studentForm.getField("name");
			tf.setRequired(true);
			tf.setCaption(app.locale.getString("GROUPMANAGERLAYOUT_GROUPSTABLE_NAME"));
			tf.setRequiredError(app.locale.getString("STUDENTSMANAGERLAYOUT_NEWSTUDENTWINDOW_NAMEREQUIREDERROR"));
			tf.addValidator(new StringLengthValidator(
					app.locale.getString("GROUPMANAGERLAYOUT_NEWGROUPWINDOW_NAMETOOLONGERROR")+ Constants.SHORT_VARCHAR_LENGTH + " " + app.locale.getString("GROUPMANAGERLAYOUT_NEWGROUPWINDOW_CHARS"),1,Constants.SHORT_VARCHAR_LENGTH,false));
					
			tf = (TextField)studentForm.getField("surname1");
			tf.setRequired(false);
			tf.setCaption(app.locale.getString("GROUPMANAGERLAYOUT_STUDENTGROUPTABLE_SURNAME1"));
			tf.addValidator(new StringLengthValidator(
					app.locale.getString("STUDENTSMANAGERLAYOUT_NEWSTUDENTWINDOW_SURNAME1TOOLONGERROR")+ Constants.SHORT_VARCHAR_LENGTH + " " + app.locale.getString("GROUPMANAGERLAYOUT_NEWGROUPWINDOW_CHARS"),1,Constants.SHORT_VARCHAR_LENGTH,false));
			
			tf = (TextField)studentForm.getField("surname2");
			tf.setRequired(false);
			tf.setCaption(app.locale.getString("GROUPMANAGERLAYOUT_STUDENTGROUPTABLE_SURNAME2"));
			tf.addValidator(new StringLengthValidator(
					app.locale.getString("STUDENTSMANAGERLAYOUT_NEWSTUDENTWINDOW_SURNAME2TOOLONGERROR")+ Constants.SHORT_VARCHAR_LENGTH + " " + app.locale.getString("GROUPMANAGERLAYOUT_NEWGROUPWINDOW_CHARS"),1,Constants.SHORT_VARCHAR_LENGTH,false));

			tf = (TextField)studentForm.getField("email");
			tf.setRequired(false);
			tf.setCaption(app.locale.getString("GROUPMANAGERLAYOUT_STUDENTGROUPTABLE_EMAIL"));
			tf.addValidator(new EmailValidator(
					app.locale.getString("STUDENTSMANAGERLAYOUT_NEWSTUDENTWINDOW_EMAILVALIDATOR")));
			tf.addValidator(new StringLengthValidator(
					app.locale.getString("STUDENTSMANAGERLAYOUT_NEWSTUDENTWINDOW_EMAILTOOLONGERROR")+ Constants.SHORT_VARCHAR_LENGTH + " " + app.locale.getString("GROUPMANAGERLAYOUT_NEWGROUPWINDOW_CHARS"),1,Constants.SHORT_VARCHAR_LENGTH,false));
			
			tf = (TextField)studentForm.getField("smsphone");
			tf.setCaption(app.locale.getString("GROUPMANAGERLAYOUT_STUDENTGROUPTABLE_SMSPHONE"));
			tf.setRequired(false);
			tf.addValidator(new RegexpValidator("[1-9][0-9]{8}",
					app.locale.getString("STUDENTSMANAGERLAYOUT_NEWSTUDENTWINDOW_SMSPHONEVALIDATOR")));

			studentForm.setImmediate(true);
		
			Button okButton = new Button(app.locale.getString("GROUPMANAGERLAYOUT_BUTTONOK_CAPTION"));
			okButton.addListener(new Button.ClickListener() {
				
				@Override
				public void buttonClick(ClickEvent event) {
					if (!studentForm.isValid()) return;
					Student student = new Student(0, 
							(String)studentForm.getField("surname1").getValue(), 
							(String)studentForm.getField("surname2").getValue(),
							(String)studentForm.getField("name").getValue(),
							(String)studentForm.getField("email").getValue(),
							(String)studentForm.getField("smsphone").getValue());
	
					Integer idnewStudent=app.storage.addStudent(student);
					refreshStudents();
					studentsTable.setValue(idnewStudent);
                    getParent().getWindow().showNotification(
                    		app.locale.getString("GROUPMANAGERLAYOUT_TOOLTIP_MESSAGE"),
                    		app.locale.getString("STUDENTSMANAGERLAYOUT_TOOLTIP_STUDENTADDED")+ " " + (String)studentForm.getField("surname1").getValue() 
                            + " " + (String)studentForm.getField("surname2").getValue(),
                            Notification.TYPE_TRAY_NOTIFICATION);
                    
					studentForm.getField("name").setValue("");
					studentForm.getField("surname1").setValue("");
					studentForm.getField("surname2").setValue("");
					studentForm.getField("email").setValue("");
					studentForm.getField("smsphone").setValue("");
                    close();
				}
			});
			okButton.setIcon(new ThemeResource("img/12-em-check.png"));
			
			Button cancelButton = new Button(app.locale.getString("GROUPMANAGERLAYOUT_BUTTONCANCEL_CAPTION"));
			cancelButton.addListener(new Button.ClickListener() {
				
				@Override
				public void buttonClick(ClickEvent event) {
					studentForm.getField("name").setValue("");
					studentForm.getField("surname1").setValue("");
					studentForm.getField("surname2").setValue("");
					studentForm.getField("email").setValue("");
					studentForm.getField("smsphone").setValue("");
                	close();
				}
			});
			cancelButton.setIcon(new ThemeResource("img/12-em-cross.png"));
			HorizontalLayout hl = new HorizontalLayout();
			studentForm.getFooter().addComponent(hl);
			hl.setSpacing(true);
			hl.addComponent(okButton);
			hl.addComponent(cancelButton);
			
    		addComponent(studentForm);
    		
    		VerticalLayout layout = (VerticalLayout) getContent();
    		layout.setSpacing(true);
    		layout.setMargin(true);
    		layout.setSizeUndefined(); // We want the window to has automatic size !!!
    		
    		setModal(true);
    		setResizable(false);
		}
		
	}
	private class EditStudentWindow extends Window {
		
		private final Form studentForm = new Form();
		private final StudentInfo studentInfo = new StudentInfo();
		private Integer idstudent;
		
		public EditStudentWindow() {
			
			studentForm.setValidationVisible(true);
			
			BeanItem<StudentInfo> groupBean = new BeanItem<StudentInfo>(studentInfo);
			studentForm.setItemDataSource(groupBean,Arrays.asList ( "name","surname1","surname2","email","smsphone"));
			
			TextField tf = (TextField)studentForm.getField("name");
			tf.setRequired(true);
			tf.setCaption(app.locale.getString("GROUPMANAGERLAYOUT_GROUPSTABLE_NAME"));
			tf.setRequiredError(app.locale.getString("STUDENTSMANAGERLAYOUT_NEWSTUDENTWINDOW_NAMEREQUIREDERROR"));
			tf.addValidator(new StringLengthValidator(
					app.locale.getString("GROUPMANAGERLAYOUT_NEWGROUPWINDOW_NAMETOOLONGERROR")+ Constants.SHORT_VARCHAR_LENGTH + " " + app.locale.getString("GROUPMANAGERLAYOUT_NEWGROUPWINDOW_CHARS"),1,Constants.SHORT_VARCHAR_LENGTH,false));
			
			tf = (TextField)studentForm.getField("surname1");
			tf.setRequired(false);
			tf.setCaption(app.locale.getString("GROUPMANAGERLAYOUT_STUDENTGROUPTABLE_SURNAME1"));
			tf.addValidator(new StringLengthValidator(
					app.locale.getString("STUDENTSMANAGERLAYOUT_NEWSTUDENTWINDOW_SURNAME1TOOLONGERROR")+ Constants.SHORT_VARCHAR_LENGTH + " " + app.locale.getString("GROUPMANAGERLAYOUT_NEWGROUPWINDOW_CHARS"),1,Constants.SHORT_VARCHAR_LENGTH,false));
			
			tf = (TextField)studentForm.getField("surname2");
			tf.setRequired(false);
			tf.setCaption(app.locale.getString("GROUPMANAGERLAYOUT_STUDENTGROUPTABLE_SURNAME2"));
			tf.addValidator(new StringLengthValidator(
					app.locale.getString("STUDENTSMANAGERLAYOUT_NEWSTUDENTWINDOW_SURNAME2TOOLONGERROR")+ Constants.SHORT_VARCHAR_LENGTH + " " + app.locale.getString("GROUPMANAGERLAYOUT_NEWGROUPWINDOW_CHARS"),1,Constants.SHORT_VARCHAR_LENGTH,false));

			tf = (TextField)studentForm.getField("email");
			tf.setRequired(false);
			tf.setCaption(app.locale.getString("GROUPMANAGERLAYOUT_STUDENTGROUPTABLE_EMAIL"));
			tf.addValidator(new EmailValidator(
					app.locale.getString("STUDENTSMANAGERLAYOUT_NEWSTUDENTWINDOW_EMAILVALIDATOR")));
			tf.addValidator(new StringLengthValidator(
					app.locale.getString("STUDENTSMANAGERLAYOUT_NEWSTUDENTWINDOW_EMAILTOOLONGERROR")+ Constants.SHORT_VARCHAR_LENGTH + " " + app.locale.getString("GROUPMANAGERLAYOUT_NEWGROUPWINDOW_CHARS"),1,Constants.SHORT_VARCHAR_LENGTH,false));
			
			tf = (TextField)studentForm.getField("smsphone");
			tf.setCaption(app.locale.getString("GROUPMANAGERLAYOUT_STUDENTGROUPTABLE_SMSPHONE"));
			tf.setRequired(false);
			tf.addValidator(new RegexpValidator("[1-9][0-9]{8}",
					app.locale.getString("STUDENTSMANAGERLAYOUT_NEWSTUDENTWINDOW_SMSPHONEVALIDATOR")));
			studentForm.setImmediate(true);
		
			Button okButton = new Button(app.locale.getString("GROUPMANAGERLAYOUT_BUTTONOK_CAPTION"));
			okButton.addListener(new Button.ClickListener() {
				
				@Override
				public void buttonClick(ClickEvent event) {
					if (!studentForm.isValid()) return;
					Integer idStudent =(Integer) studentsTable.getValue();
					Student student = new Student(idStudent, 
							(String)studentForm.getField("surname1").getValue(), 
							(String)studentForm.getField("surname2").getValue(),
							(String)studentForm.getField("name").getValue(),
							(String)studentForm.getField("email").getValue(),
							(String)studentForm.getField("smsphone").getValue());

					app.storage.updateStudent(student);
                    getParent().getWindow().showNotification(
                    		app.locale.getString("GROUPMANAGERLAYOUT_TOOLTIP_MESSAGE"),
                    		app.locale.getString("STUDENTSMANAGERLAYOUT_TOOLTIP_STUDENTEDITED")+ " " + (String)studentForm.getField("surname1").getValue() 
                            + " " + (String)studentForm.getField("surname2").getValue(),
                            Notification.TYPE_TRAY_NOTIFICATION);
                    
					studentForm.getField("name").setValue("");
					studentForm.getField("surname1").setValue("");
					studentForm.getField("surname2").setValue("");
					studentForm.getField("email").setValue("");
					studentForm.getField("smsphone").setValue("");
					refreshStudents();
					studentsTable.setValue(idStudent);
					close();
				}
			});
			okButton.setIcon(new ThemeResource("img/12-em-check.png"));
			
			Button cancelButton = new Button(app.locale.getString("GROUPMANAGERLAYOUT_BUTTONCANCEL_CAPTION"));
			cancelButton.addListener(new Button.ClickListener() {
				
				@Override
				public void buttonClick(ClickEvent event) {
					studentForm.getField("name").setValue("");
					studentForm.getField("surname1").setValue("");
					studentForm.getField("surname2").setValue("");
					studentForm.getField("email").setValue("");
					studentForm.getField("smsphone").setValue("");
					close();
				}
			});
			cancelButton.setIcon(new ThemeResource("img/12-em-cross.png"));
			HorizontalLayout hl = new HorizontalLayout();
			studentForm.getFooter().addComponent(hl);
			hl.setSpacing(true);
			hl.addComponent(okButton);
			hl.addComponent(cancelButton);
			
    		addComponent(studentForm);
    		
    		VerticalLayout layout = (VerticalLayout) getContent();
    		layout.setSpacing(true);
    		layout.setMargin(true);
    		layout.setSizeUndefined(); // We want the window to has automatic size !!!
    		
    		setModal(true);
    		setResizable(false);
		}
		public void setIdStudent(Integer idstudent){
			this.idstudent=idstudent;
		}
		
	}
	private class DeleteStudentWindow extends Window {
		
		private Student student=new Student(0,"","","","","");
		private Label messageLabel;

		public DeleteStudentWindow() {
			Embedded warningEmbedded = new Embedded(null,new ThemeResource("img/Warning_icon.png"));
			 
			messageLabel = new Label();
			String messageString = app.locale.getString("STUDENTSMANAGERLAYOUT_DELETEWINDOW_MESSAGE");

			messageLabel.setValue(messageString);
			messageLabel.setStyleName("h3");
			messageLabel.setContentMode(Label.CONTENT_XHTML);
			
			HorizontalLayout warningLayout = new HorizontalLayout();
			warningLayout.setSpacing(true);
			warningLayout.addComponent(warningEmbedded);
			warningLayout.addComponent(messageLabel);
			warningLayout.setSizeFull();
			
			Button okButton = new Button(app.locale.getString("GROUPMANAGERLAYOUT_BUTTONOK_CAPTION"));
			okButton.addListener(new Button.ClickListener() {
				
				@Override
				public void buttonClick(ClickEvent event) {
					app.storage.deleteStudent(student);
                    getParent().getWindow().showNotification(
                    		app.locale.getString("GROUPMANAGERLAYOUT_TOOLTIP_MESSAGE"),
                    		app.locale.getString("STUDENTSMANAGERLAYOUT_TOOLTIP_STUDENTDELETED")+ " " +
                    		(String)student.getName() +
                            (String)student.getSurname1() +
                            (String)student.getSurname2(),
                            Notification.TYPE_TRAY_NOTIFICATION);
					refreshStudents();
					close();
				}
			});
			okButton.setIcon(new ThemeResource("img/12-em-check.png"));
			
			Button cancelButton = new Button(app.locale.getString("GROUPMANAGERLAYOUT_BUTTONCANCEL_CAPTION"));
			cancelButton.addListener(new Button.ClickListener() {
				
				@Override
				public void buttonClick(ClickEvent event) {
					close();
				}
			});
			cancelButton.setIcon(new ThemeResource("img/12-em-cross.png"));
			HorizontalLayout hl = new HorizontalLayout();
			hl.setSpacing(true);
			hl.setMargin(true);
			hl.addComponent(okButton);
			hl.addComponent(cancelButton);
			hl.setSizeFull();
			addComponent(warningLayout);
    		addComponent(hl);
    		
    		VerticalLayout layout = (VerticalLayout) getContent();
    		layout.setSpacing(true);
    		layout.setMargin(true);
    		layout.setSizeUndefined(); // We want the window to has automatic size !!!
    		
    		setModal(true);
    		setResizable(false);
		}
		public void setStudent(Student student){
			this.student=student;
			
		}
		
	}
	
	/**
	 * Ventana emergente de importación de datos en formato csv.
	 * 
	 * @author apalacios
	 *
	 */
	private class ImportStudentWindow extends Window implements  Upload.FailedListener, Upload.SucceededListener {
		
		private CSVUploader csvuploader = new CSVUploader(Student.class.getConstructor(String.class, String.class, String.class, String.class, String.class), Student.getBasicProperties());		
		private Upload upload=new Upload(app.locale.getString("GROUPMANAGERLAYOUT_UPLOAD_CAPTION"),csvuploader);
		
		/**
		 * Constructor de la ventana de importación de csv. Crea un objeto Upload y lo conecta con el CSVUploader.
		 * @throws SecurityException
		 * @throws NoSuchMethodException
		 */
		public ImportStudentWindow() throws SecurityException, NoSuchMethodException {

			String alert = "<p><br/>Asegúrese de que la estructura de los registros del CSV sea la siguiente:</p>" + 
			"<table border=\"1\" align=\"center\">" +
			"<tr>" + "<th> surname1</th>" + "<th> surname2</th>" + "<th> name</th>" + "<th> email</th>" + "<th> smsphone</th>" + "</tr>" +
			"<tr>" + "<th> \"student1\"</th>" + "<th> \"student1\"</th>" + "<th> \"student1\"</th>" + "<th> \"student1@mail.com\"</th>" + "<th> \"666666666\"</th>" +
			"</tr>" + "</table>";
			
			csvuploader.setAlert(alert);
			
			addComponent(upload);
			upload.setButtonCaption(app.locale.getString("GROUPMANAGERLAYOUT_UPLOADBUTTON_CAPTION"));
			
			upload.addListener((Upload.FinishedListener)csvuploader);
			upload.addListener((Upload.SucceededListener)this);
			upload.addListener((Upload.FailedListener)this);
						
			Button cancelButton = new Button(app.locale.getString("GROUPMANAGERLAYOUT_BUTTONCANCEL_CAPTION"));
			cancelButton.addListener(new Button.ClickListener() {
				
				@Override
				public void buttonClick(ClickEvent event) {
					close();
				}
			});
			cancelButton.setIcon(new ThemeResource("img/12-em-cross.png"));
			HorizontalLayout hl = new HorizontalLayout();
			hl.setSpacing(true);
			hl.setMargin(true);
			hl.addComponent(cancelButton);
    		addComponent(hl);
    		
    		VerticalLayout layout = (VerticalLayout) getContent();
    		layout.setSpacing(true);
    		layout.setMargin(true);
    		layout.setSizeUndefined(); // We want the window to has automatic size !!!
    		
    		setModal(true);
    		setResizable(false);
		}

		/**
		 * Evento de carga correcta. Muestra una notificación con el número de registros correctos y fallidos.
		 */
		@Override
		public void uploadSucceeded(SucceededEvent event) {
			refreshStudents();
			close();
			System.out.println("Alumnos cargados !!!");
			
			String finishMessage = app.locale.getString("GROUPMANAGERLAYOUT_UPLOADSUCCESS_TAGMESSAGE")+" "
				+ csvuploader.successfulRecordsCounter + " "
				+ app.locale.getString("GROUPMANAGERLAYOUT_UPLOADSUCCESSOK_MESSAGE")
				+ app.locale.getString("GROUPMANAGERLAYOUT_UPLOADSUCCESS_TAGMESSAGE")+" "
				+ csvuploader.failedRecordsCounter + " "
				+ app.locale.getString("GROUPMANAGERLAYOUT_UPLOADSUCCESSNOK_MESSAGE");
		
			Label finishLabel = csvuploader.getLabelFromString(finishMessage);			
			AttendancemanagerApplication.adminWindow.showNotification("CARGA FINALIZADA",(String) finishLabel.getValue(), Notification.TYPE_TRAY_NOTIFICATION);
		}
		
		/**
		 * Evento de carga fallida. Si la carga falla, muestra una alerta de error.
		 */
		@Override
		public void uploadFailed(FailedEvent event) {
			System.out.println("Ohhhh, no he podido insertar los alumnos !!!");
			
			String errorMessage = app.locale.getString("GROUPMANAGERLAYOUT_UPLOADFAILED_ERRORSTRING");
			Label errorLabel = csvuploader.getLabelFromString(errorMessage);
			AttendancemanagerApplication.adminWindow.showNotification("ERROR",(String) errorLabel.getValue(), Notification.TYPE_ERROR_MESSAGE);
		}		
	}

}
