package org.iespuigcastellar.attendancemanager.screenlayouts;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Arrays;
import java.util.Date;

import org.iespuigcastellar.attendancemanager.AttendancemanagerApplication;
import org.iespuigcastellar.attendancemanager.core.CSVUploader;
import org.iespuigcastellar.attendancemanager.core.Constants;
import org.iespuigcastellar.attendancemanager.core.DatabaseStorage;
import org.iespuigcastellar.attendancemanager.core.Subject;
import org.iespuigcastellar.attendancemanager.core.Teacher;
import org.iespuigcastellar.attendancemanager.core.Term;

import com.vaadin.data.Item;
import com.vaadin.data.Property;
import com.vaadin.data.Property.ValueChangeEvent;
import com.vaadin.data.util.BeanItem;
import com.vaadin.data.validator.StringLengthValidator;
import com.vaadin.terminal.ThemeResource;
import com.vaadin.ui.Button;
import com.vaadin.ui.ComboBox;
import com.vaadin.ui.Component;
import com.vaadin.ui.CssLayout;
import com.vaadin.ui.DefaultFieldFactory;
import com.vaadin.ui.Embedded;
import com.vaadin.ui.Field;
import com.vaadin.ui.Form;
import com.vaadin.ui.HorizontalLayout;
import com.vaadin.ui.Label;
import com.vaadin.ui.Panel;
import com.vaadin.ui.Table;
import com.vaadin.ui.TextField;
import com.vaadin.ui.Upload;
import com.vaadin.ui.VerticalLayout;
import com.vaadin.ui.Window;
import com.vaadin.ui.Button.ClickEvent;
import com.vaadin.ui.Upload.FailedEvent;
import com.vaadin.ui.Upload.SucceededEvent;
import com.vaadin.ui.Window.Notification;


public class SubjectManagerLayout extends VerticalLayout{
	private AttendancemanagerApplication app;
	
	private Button newButton;
	private Button editButton;
	private Button deleteButton;
	private Button importButton;
	
	private Table subjectsTable;
	
	private NewSubjectWindow newSubjectWindow;
	private EditSubjectWindow editSubjectWindow;
	private DeleteSubjectWindow deleteSubjectWindow;
	private ImportSubjectWindow importSubjectWindow;
	
	private void initLayout() {
		newButton = new Button();
		newButton.setCaption(app.locale.getString("GROUPMANAGERLAYOUT_NEWBUTTON_CAPTION"));
		newButton.setIcon(new ThemeResource("img/book_add.png"));
		newButton.setStyleName("default");

		
		editButton = new Button();
		editButton.setCaption(app.locale.getString("GROUPMANAGERLAYOUT_EDITBUTTON_CAPTION"));
		editButton.setIcon(new ThemeResource("img/book_edit.png"));
		editButton.setEnabled(false);
		editButton.setStyleName("default");

		deleteButton = new Button();
		deleteButton.setCaption(app.locale.getString("GROUPMANAGERLAYOUT_DELETEBUTTON_CAPTION"));
		deleteButton.setIcon(new ThemeResource("img/book_delete.png"));
		deleteButton.setEnabled(false);
		deleteButton.setStyleName("default");

		importButton = new Button();
		importButton.setCaption(app.locale.getString("GROUPMANAGERLAYOUT_IMPORTBUTTON_CAPTION"));
		importButton.setIcon(new ThemeResource("img/book_go.png"));
		importButton.setStyleName("default");
		
		CssLayout toolbarsubjects = new CssLayout();
		toolbarsubjects.addComponent(newButton);
		toolbarsubjects.addComponent(editButton);
		toolbarsubjects.addComponent(deleteButton);
		toolbarsubjects.addComponent(importButton);
		toolbarsubjects.setStyleName("toolbar");
		toolbarsubjects.setWidth("100%");
		
		subjectsTable = new Table();
		subjectsTable.setImmediate(true);
		subjectsTable.setColumnReorderingAllowed(true);
		subjectsTable.setColumnCollapsingAllowed(true);
		subjectsTable.setStyleName("striped");
		subjectsTable.setSizeFull();
				
		Panel subjectsPanel = new Panel();
		subjectsPanel.setCaption(app.locale.getString("SUBJECTSMANAGERLAYOUT_SUBJECTSPANEL_CAPTION"));
		subjectsPanel.addComponent(subjectsTable);
		subjectsPanel.setSizeFull();
		subjectsPanel.setStyleName("bubble");
		subjectsPanel.setIcon(new ThemeResource("img/book.png"));
		subjectsPanel.getContent().setSizeFull();
			
		
		VerticalLayout vl =  (VerticalLayout) subjectsPanel.getContent();
		vl.setMargin(false);
		vl.setExpandRatio(subjectsTable, 1.0f);
		vl.setSpacing(false);
		
		addComponent(toolbarsubjects);
		addComponent(subjectsPanel);
		
		setSizeFull();
		setMargin(false, true, false, true);
		
		setExpandRatio(subjectsPanel, 1.0f);
		setSpacing(true);
	}

	public void attach() {

		app = (AttendancemanagerApplication)getApplication();
		initLayout();
		
		subjectsTable.addContainerProperty(DatabaseStorage.NAME_PROPERTY, String.class, "");
		subjectsTable.addContainerProperty(DatabaseStorage.COMMENT_PROPERTY, String.class, "");
		
		app.storage.refreshTable(app.storage.getSubjects(), subjectsTable);
		String visiblecolumns[]={DatabaseStorage.NAME_PROPERTY,DatabaseStorage.COMMENT_PROPERTY};
		subjectsTable.setColumnHeaders(new String[] { app.locale.getString("GROUPMANAGERLAYOUT_GROUPSTABLE_NAME"),app.locale.getString("GROUPMANAGERLAYOUT_GROUPSTABLE_COMMENT")});
		subjectsTable.setVisibleColumns(visiblecolumns);
		subjectsTable.setSelectable(true);
		subjectsTable.setMultiSelect(false);
		subjectsTable.setImmediate(true);

		subjectsTable.addListener(new Property.ValueChangeListener() {
			
			@Override
			public void valueChange(ValueChangeEvent event) {
				Object value = subjectsTable.getValue();
				if (value!=null){
					editButton.setEnabled(true);
					deleteButton.setEnabled(true);
				} else {
					editButton.setEnabled(false);
					deleteButton.setEnabled(false);
					
				}
			}
		});

		newSubjectWindow = new NewSubjectWindow();
		newSubjectWindow.setCaption(app.locale.getString("SUBJECTSMANAGERLAYOUT_NEWSUBJECTWINDOW_CAPTION"));
		newButton.addListener(new Button.ClickListener() {
			
			@Override
			public void buttonClick(ClickEvent event) {
				// Open window if not open already
				if(newSubjectWindow.getParent() != null) {
					// Window already open
				} else {
					// open window
					getWindow().addWindow(newSubjectWindow);
				}
			}
		});
		editSubjectWindow = new EditSubjectWindow();
		editSubjectWindow.setCaption("EDIT A SUBJECT");
		editButton.addListener(new Button.ClickListener() {
			
			@Override
			public void buttonClick(ClickEvent event) {
				// TODO Auto-generated method stub
				Integer idItem=(Integer) subjectsTable.getValue();
				Item item=subjectsTable.getItem(idItem);
				Integer idSubject=(Integer) subjectsTable.getValue();
				String name=(String) item.getItemProperty("name").getValue();
				String comment=(String) item.getItemProperty("comment").getValue();
				
				Subject subject = new Subject(idSubject,0,name,comment);
				editSubjectWindow.setSubject(subject);
				getWindow().addWindow(editSubjectWindow);
			}
		});	
		deleteSubjectWindow = new DeleteSubjectWindow();
		deleteSubjectWindow.setCaption("DELETE A SUBJECT");
		deleteButton.addListener(new Button.ClickListener() {
			
			@Override
			public void buttonClick(ClickEvent event) {
				// TODO Auto-generated method stub
				Integer idItem=(Integer) subjectsTable.getValue();
				Item item=subjectsTable.getItem(idItem);
				Integer idSubject=(Integer) subjectsTable.getValue();
				String name=(String) item.getItemProperty(DatabaseStorage.NAME_PROPERTY).getValue();
				String comment=(String) item.getItemProperty(DatabaseStorage.COMMENT_PROPERTY).getValue();
				Subject subject = new Subject(idSubject,0,name,comment);
				deleteSubjectWindow.setSubject(subject);
				getWindow().addWindow(deleteSubjectWindow);
			}
		});
		try {
			importSubjectWindow = new ImportSubjectWindow();
		} catch (SecurityException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (NoSuchMethodException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		importSubjectWindow.setCaption("IMPORT FILE");
		importButton.addListener(new Button.ClickListener() {
			
			@Override
			public void buttonClick(ClickEvent event) {
				// TODO Auto-generated method stub
				getWindow().addWindow(importSubjectWindow);
			}
		});	
	}
	
	public void refreshSubjects() {
		app.storage.refreshTable(app.storage.getSubjects(), subjectsTable);
	}
	
	private class NewSubjectWindow extends Window {
		
		private final Form subjectForm = new Form();
		private SubjectInfo subjectInfo = new SubjectInfo();
		
		public NewSubjectWindow() {
			

	    	subjectInfo = new SubjectInfo(); // a person POJO
	        BeanItem subjectItem = new BeanItem(subjectInfo); // item from POJO

	        // Create the Form;
	        subjectForm.setCaption(app.locale.getString("SUBJECTSMANAGERLAYOUT_SUBJECTFORM_CAPTION"));
	        subjectForm.setWriteThrough(false); // we want explicit 'apply'
	        subjectForm.setInvalidCommitted(false); // no invalid values in datamodel

	        // FieldFactory for customizing the fields and adding validators
	        subjectForm.setFormFieldFactory(new SubjectFieldFactory());
	        subjectForm.setItemDataSource(subjectItem); // bind to POJO via BeanItem

	        // Determines which properties are shown, and in which order:
	        subjectForm.setVisibleItemProperties(Arrays.asList(new String[] {
	                "name", "comment"}));

			Button okButton = new Button(app.locale.getString("GROUPMANAGERLAYOUT_BUTTONOK_CAPTION"));
			okButton.addListener(new Button.ClickListener() {
				
				@Override
				public void buttonClick(ClickEvent event) {
					if (!subjectForm.isValid()) return;
					Subject subject = new Subject(0,0,
							(String)subjectForm.getField("name").getValue(), 
							(String)subjectForm.getField("comment").getValue());	
					
					Integer idnewSubject=app.storage.addSubject(subject);
                    getParent().getWindow().showNotification(
                    		app.locale.getString("GROUPMANAGERLAYOUT_TOOLTIP_MESSAGE"),
                    		app.locale.getString("SUBJECTSMANAGERLAYOUT_TOOLTIP_SUBJECTADDED")+ " " +(String)subjectForm.getField("name").getValue(),
                            Notification.TYPE_TRAY_NOTIFICATION);
					setValuesEmpty();
					refreshSubjects();
					subjectsTable.setValue(idnewSubject);
                    close();
				}
			});
			okButton.setIcon(new ThemeResource("img/12-em-check.png"));
			
			Button cancelButton = new Button(app.locale.getString("GROUPMANAGERLAYOUT_BUTTONCANCEL_CAPTION"));
			cancelButton.addListener(new Button.ClickListener() {
				
				@Override
				public void buttonClick(ClickEvent event) {   
					setValuesEmpty();
                	close();
				}
			});
			cancelButton.setIcon(new ThemeResource("img/12-em-cross.png"));
			HorizontalLayout hl = new HorizontalLayout();
			subjectForm.getFooter().addComponent(hl);
			
			hl.setSpacing(true);
			hl.addComponent(okButton);
			hl.addComponent(cancelButton);
    		addComponent(subjectForm);
    		
    		VerticalLayout layout = (VerticalLayout) getContent();
    		layout.setSpacing(true);
    		layout.setMargin(true);
    		layout.setSizeUndefined(); // We want the window to has automatic size !!!
    		
    		setModal(true);
    		setResizable(false);

		}
		private void setValuesEmpty(){
			
			subjectForm.getField("name").setValue("");
			subjectForm.getField("comment").setValue("");
		}
	}

	private class EditSubjectWindow extends Window {
		
		private final Form subjectForm = new Form();
		private SubjectInfo subjectInfo = new SubjectInfo();
		private Subject subject;
		
		public EditSubjectWindow() {
			
		subject=new Subject(0,0,"","");
	    	subjectInfo = new SubjectInfo(); // a person POJO
	        BeanItem subjectItem = new BeanItem(subjectInfo); // item from POJO

	        // Create the Form;
	        subjectForm.setCaption("Subject details");

	        subjectForm.setCaption(app.locale.getString("SUBJECTSMANAGERLAYOUT_SUBJECTFORM_CAPTION"));

	        subjectForm.setWriteThrough(false); // we want explicit 'apply'
	        subjectForm.setInvalidCommitted(false); // no invalid values in datamodel

	        // FieldFactory for customizing the fields and adding validators
	        subjectForm.setFormFieldFactory(new SubjectFieldFactory());
	        subjectForm.setItemDataSource(subjectItem); // bind to POJO via BeanItem

	        // Determines which properties are shown, and in which order:
	        subjectForm.setVisibleItemProperties(Arrays.asList(new String[] {
	                "name", "comment"}));


			Button okButton = new Button(app.locale.getString("GROUPMANAGERLAYOUT_BUTTONOK_CAPTION"));

			okButton.addListener(new Button.ClickListener() {
				
				@Override
				public void buttonClick(ClickEvent event) {
					if (!subjectForm.isValid()) return;
					Subject editedsubject = new Subject(subject.getID(),0,
							(String)subjectForm.getField("name").getValue(), 
							(String)subjectForm.getField("comment").getValue());	
					
					app.storage.updateSubject(editedsubject);
                    getParent().getWindow().showNotification(

                            app.locale.getString("GROUPMANAGERLAYOUT_TOOLTIP_MESSAGE"),
                            app.locale.getString("SUBJECTSMANAGERLAYOUT_TOOLTIP_SUBJECTEDITED") + " " + (String)subjectForm.getField("name").getValue(),

                            Notification.TYPE_TRAY_NOTIFICATION);
					setValuesEmpty();
					refreshSubjects();
					subjectsTable.setValue(subject.getID());
                    close();
				}
			});
			okButton.setIcon(new ThemeResource("img/12-em-check.png"));
			

			Button cancelButton = new Button(app.locale.getString("GROUPMANAGERLAYOUT_BUTTONCANCEL_CAPTION"));

			cancelButton.addListener(new Button.ClickListener() {
				
				@Override
				public void buttonClick(ClickEvent event) {   
					setValuesEmpty();
                	close();
				}
			});
			cancelButton.setIcon(new ThemeResource("img/12-em-cross.png"));
			HorizontalLayout hl = new HorizontalLayout();
			subjectForm.getFooter().addComponent(hl);
			
			hl.setSpacing(true);
			hl.addComponent(okButton);
			hl.addComponent(cancelButton);
    		addComponent(subjectForm);
    		
    		VerticalLayout layout = (VerticalLayout) getContent();
    		layout.setSpacing(true);
    		layout.setMargin(true);
    		layout.setSizeUndefined(); // We want the window to has automatic size !!!
    		
    		setModal(true);
    		setResizable(false);

		}
		private void setValuesEmpty(){
			
			subjectForm.getField("name").setValue("");
			subjectForm.getField("comment").setValue("");
		}
		private void setSubject(Subject subject){
			this.subject=subject;
			subjectForm.getField("name").setValue(subject.getName());
			subjectForm.getField("comment").setValue(subject.getComment());

		}
	}

	private class DeleteSubjectWindow extends Window{
		
		private Subject subject;
		private Label messageLabel;
		
		public DeleteSubjectWindow() {
			
			Embedded warningEmbedded = new Embedded(null,new ThemeResource("img/Warning_icon.png"));
			 
			messageLabel = new Label();

			String messageString = app.locale.getString("SUBJECTSMANAGERLAYOUT_DELETEWINDOW_MESSAGE");

			
			messageLabel.setValue(messageString);
			messageLabel.setContentMode(Label.CONTENT_XHTML);
			
			HorizontalLayout warningLayout = new HorizontalLayout();
			warningLayout.setSpacing(true);
			warningLayout.addComponent(warningEmbedded);
			warningLayout.addComponent(messageLabel);
			warningLayout.setSizeFull();

			Button okButton = new Button(app.locale.getString("GROUPMANAGERLAYOUT_BUTTONOK_CAPTION"));
			okButton.addListener(new Button.ClickListener() {
				
				@Override
				public void buttonClick(ClickEvent event) {									
					app.storage.deleteSubject(subject.getID());
                    getParent().getWindow().showNotification(
                            app.locale.getString("GROUPMANAGERLAYOUT_TOOLTIP_MESSAGE"),
                            app.locale.getString("SUBJECTSMANAGERLAYOUT_TOOLTIP_SUBJECTDELETED") + " " + (String)subject.getName(),

                            Notification.TYPE_TRAY_NOTIFICATION);
					refreshSubjects();
					close();
				}
			});
			okButton.setIcon(new ThemeResource("img/12-em-check.png"));
			

			Button cancelButton = new Button(app.locale.getString("GROUPMANAGERLAYOUT_BUTTONCANCEL_CAPTION"));

			cancelButton.addListener(new Button.ClickListener() {
				
				@Override
				public void buttonClick(ClickEvent event) {
					close();
				}
			});
			cancelButton.setIcon(new ThemeResource("img/12-em-cross.png"));
			HorizontalLayout hl = new HorizontalLayout();
			hl.setSpacing(true);
			hl.setMargin(true);
			hl.addComponent(okButton);
			hl.addComponent(cancelButton);
			hl.setSizeFull();
			addComponent(warningLayout);
    		addComponent(hl);
    		
    		VerticalLayout layout = (VerticalLayout) getContent();
    		layout.setSpacing(true);
    		layout.setMargin(true);
    		layout.setSizeUndefined(); // We want the window to has automatic size !!!
    		
    		setModal(true);
    		setResizable(false);
		}

		private void setSubject(Subject subject){
			this.subject=subject;
		}
	}

	/**
	 * Ventana emergente de importación de datos en formato csv.
	 * 
	 * @author apalacios
	 *
	 */
	private class ImportSubjectWindow extends Window implements Upload.SucceededListener, Upload.FailedListener{

		private CSVUploader csvuploader = new CSVUploader(Subject.class.getConstructor(String.class, String.class), Subject.getBasicProperties());
		private Upload upload=new Upload(app.locale.getString("GROUPMANAGERLAYOUT_UPLOAD_CAPTION"),csvuploader);
		
		/**
		 * Constructor de la ventana de importación de csv. Crea un objeto Upload y lo conecta con el CSVUploader.
		 * @throws SecurityException
		 * @throws NoSuchMethodException
		 */
		public ImportSubjectWindow() throws SecurityException, NoSuchMethodException {

			String alert = "<p><br/>Asegúrese de que la estructura de los registros del CSV sea la siguiente:</p>" + 
			"<table border=\"1\" align=\"center\">" +
			"<tr>" + "<th> name</th>" + "<th> comment</th>" + "</tr>" +
			"<tr>" + "<th> \"subject1\"</th>" + "<th> \"subject1\"</th>" +
			"</tr>" + "</table>";
			
			csvuploader.setAlert(alert);
			
			addComponent(upload);
			upload.setButtonCaption(app.locale.getString("GROUPMANAGERLAYOUT_UPLOADBUTTON_CAPTION"));
			
			upload.addListener((Upload.FinishedListener)csvuploader);
			upload.addListener((Upload.SucceededListener)this);
			upload.addListener((Upload.FailedListener)this);
						
			Button cancelButton = new Button(app.locale.getString("GROUPMANAGERLAYOUT_BUTTONCANCEL_CAPTION"));

			cancelButton.addListener(new Button.ClickListener() {
				
				@Override
				public void buttonClick(ClickEvent event) {
					close();
				}
			});
			cancelButton.setIcon(new ThemeResource("img/12-em-cross.png"));
			HorizontalLayout hl = new HorizontalLayout();
			hl.setSpacing(true);
			hl.setMargin(true);
			hl.addComponent(cancelButton);
    		addComponent(hl);
    		
    		VerticalLayout layout = (VerticalLayout) getContent();
    		layout.setSpacing(true);
    		layout.setMargin(true);
    		layout.setSizeUndefined(); // We want the window to has automatic size !!!
    		
    		setModal(true);
    		setResizable(false);
		}
		
		/**
		 * Evento de carga correcta. Muestra una notificación con el número de registros correctos y fallidos.
		 */
		@Override
		public void uploadSucceeded(SucceededEvent event) {
			refreshSubjects();
			close();
			System.out.println("Asignaturas cargadas !!!");
			
			String finishMessage = app.locale.getString("GROUPMANAGERLAYOUT_UPLOADSUCCESS_TAGMESSAGE")+" "
				+ csvuploader.successfulRecordsCounter + " "
				+ app.locale.getString("GROUPMANAGERLAYOUT_UPLOADSUCCESSOK_MESSAGE")
				+ app.locale.getString("GROUPMANAGERLAYOUT_UPLOADSUCCESS_TAGMESSAGE")+" "
				+ csvuploader.failedRecordsCounter + " "
				+ app.locale.getString("GROUPMANAGERLAYOUT_UPLOADSUCCESSNOK_MESSAGE");
		
			Label finishLabel = csvuploader.getLabelFromString(finishMessage);
			AttendancemanagerApplication.adminWindow.showNotification("CARGA FINALIZADA",(String) finishLabel.getValue(), Notification.TYPE_TRAY_NOTIFICATION);
		}
		
		/**
		 * Evento de carga fallida. Si la carga falla, muestra una alerta de error.
		 */
		@Override
		public void uploadFailed(FailedEvent event) {
			System.out.println("Ohhhh, no he podido insertar las asignaturas !!!");
			
			String errorMessage = app.locale.getString("GROUPMANAGERLAYOUT_UPLOADFAILED_ERRORSTRING");
			Label errorLabel = csvuploader.getLabelFromString(errorMessage);
			AttendancemanagerApplication.adminWindow.showNotification("ERROR",(String) errorLabel.getValue(), Notification.TYPE_ERROR_MESSAGE);
		}
	}
	
	private class SubjectFieldFactory extends DefaultFieldFactory {
		
	    @Override
	    public Field createField(Item item, Object propertyId,
	            Component uiContext) {
	        Field f = super.createField(item, propertyId, uiContext);
	        if ("name".equals(propertyId)) {
	            TextField tf = (TextField) f;
	            tf.setRequired(true);
	            tf.setCaption(app.locale.getString("GROUPMANAGERLAYOUT_GROUPSTABLE_NAME"));
	            tf.setRequiredError(app.locale.getString("SUBJECTSMANAGERLAYOUT_NEWSUBJECTWINDOW_NAMEREQUIREDERROR"));
	            tf.addValidator(new StringLengthValidator(
	                    app.locale.getString("SUBJECTSMANAGERLAYOUT_NEWSUBJECTWINDOW_NAMELONGERROR") + " " + app.locale.getString("GROUPMANAGERLAYOUT_NEWGROUPWINDOW_CHARS"), 3, 25, false));
	        } else if ("comment".equals(propertyId)) {
	            TextField tf = (TextField) f;
	            tf.setCaption(app.locale.getString("GROUPMANAGERLAYOUT_GROUPSTABLE_COMMENT"));
	            tf.setRequiredError(app.locale.getString("SUBJECTSMANAGERLAYOUT_NEWSUBJECTWINDOW_COMMENTREQUIREDERROR"));
	            tf.addValidator(new StringLengthValidator(
	            		app.locale.getString("SUBJECTSMANAGERLAYOUT_NEWSUBJECTWINDOW_COMMENTLONGERROR") + " " + app.locale.getString("GROUPMANAGERLAYOUT_NEWGROUPWINDOW_CHARS"), 3, 50, false));
	        } 
	        return f;
	    }
	}

	public class SubjectInfo {

		private String name = "";
		private String comment = "";
		public String getName() {
			return name;
		}
		public void setName(String name) {	
			this.name = name;
		}
		public String getComment() {
			return comment;
		}
		public void setComment(String comment) {
			this.comment = comment;
		}
	}
}
