  /* AttendanceManager
    Copyright (C) 2010  Victor Carceler

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Affero General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Affero General Public License for more details.

    You should have received a copy of the GNU Affero General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>
 */

/*
 *	GroupManagerLayout designed by Xavier Junyent <xavierjunyent@yahoo.es>
 *
 *	GroupManagerLayout allows group administration.
 */

package org.iespuigcastellar.attendancemanager.screenlayouts;

import java.sql.SQLException;
import java.util.Arrays;
import java.util.Collection;
import java.util.Date;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Set;

import org.iespuigcastellar.attendancemanager.AttendancemanagerApplication;
import org.iespuigcastellar.attendancemanager.core.CSVUploader;
import org.iespuigcastellar.attendancemanager.core.Constants;
import org.iespuigcastellar.attendancemanager.core.DatabaseStorage;
import org.iespuigcastellar.attendancemanager.core.Group;
import org.iespuigcastellar.attendancemanager.core.Student;
import org.iespuigcastellar.attendancemanager.core.Teacher;

import com.vaadin.Application;
import com.vaadin.data.Item;
import com.vaadin.data.Property;
import com.vaadin.data.Property.ValueChangeEvent;
import com.vaadin.data.util.BeanItem;
import com.vaadin.data.util.QueryContainer;
import com.vaadin.data.validator.AbstractStringValidator;
import com.vaadin.data.validator.EmailValidator;
import com.vaadin.data.validator.RegexpValidator;
import com.vaadin.data.validator.StringLengthValidator;
import com.vaadin.terminal.ThemeResource;
import com.vaadin.ui.Alignment;
import com.vaadin.ui.Button;
import com.vaadin.ui.CheckBox;
import com.vaadin.ui.Component;
import com.vaadin.ui.CssLayout;
import com.vaadin.ui.Embedded;
import com.vaadin.ui.Form;
import com.vaadin.ui.HorizontalLayout;
import com.vaadin.ui.Label;
import com.vaadin.ui.Panel;
import com.vaadin.ui.SplitPanel;
import com.vaadin.ui.Table;
import com.vaadin.ui.TextField;
import com.vaadin.ui.Upload;
import com.vaadin.ui.VerticalLayout;
import com.vaadin.ui.Window;
import com.vaadin.ui.Button.ClickEvent;
import com.vaadin.ui.Upload.FailedEvent;
import com.vaadin.ui.Upload.SucceededEvent;
import com.vaadin.ui.Window.Notification;


@SuppressWarnings("serial")
public class TeachersManagerLayout extends VerticalLayout {
	
	private AttendancemanagerApplication app;
	
	private Button newButton;
	private Button editButton;
	private Button deleteButton;
	private Button importButton;
	
	private Table teachersTable;

	
	private NewTeacherWindow newTeacherWindow;
	private EditTeacherWindow editTeachersWindow;
	private DeleteTeacherWindow deleteTeachersWindow;
	private ImportTeacherWindow importTeachersWindow;

	
	private void initLayout() {
		
		newButton = new Button();
		newButton.setCaption(app.locale.getString("GROUPMANAGERLAYOUT_NEWBUTTON_CAPTION"));
		newButton.setIcon(new ThemeResource("img/user_add.png"));
		newButton.setStyleName("default");
		
	
		editButton = new Button();
		editButton.setCaption(app.locale.getString("GROUPMANAGERLAYOUT_EDITBUTTON_CAPTION"));
		editButton.setIcon(new ThemeResource("img/user_edit.png"));
		editButton.setEnabled(false);
		editButton.setStyleName("default");
		
		deleteButton = new Button();
		deleteButton.setCaption(app.locale.getString("GROUPMANAGERLAYOUT_DELETEBUTTON_CAPTION"));
		deleteButton.setIcon(new ThemeResource("img/user_delete.png"));
		deleteButton.setEnabled(false);
		deleteButton.setStyleName("default");
		
		importButton = new Button();
		importButton.setCaption(app.locale.getString("GROUPMANAGERLAYOUT_IMPORTBUTTON_CAPTION"));
		importButton.setIcon(new ThemeResource("img/user_go.png"));
		importButton.setStyleName("default");
		
		CssLayout toolbarteachers = new CssLayout();
		toolbarteachers.addComponent(newButton);
		toolbarteachers.addComponent(editButton);
		toolbarteachers.addComponent(deleteButton);
		toolbarteachers.addComponent(importButton);
		toolbarteachers.setStyleName("toolbar");
		toolbarteachers.setWidth("100%");
		
		teachersTable = new Table();
		teachersTable.setImmediate(true);
		teachersTable.setColumnReorderingAllowed(true);
		teachersTable.setColumnCollapsingAllowed(true);
		teachersTable.setStyleName("striped");
		teachersTable.setSizeFull();
				
		Panel teachersPanel = new Panel();
		teachersPanel.setCaption(app.locale.getString("TEACHERSMANAGERLAYOUT_TEACHERSPANEL_CAPTION"));
		teachersPanel.addComponent(teachersTable);
		teachersPanel.setSizeFull();
		teachersPanel.setStyleName("bubble");
		teachersPanel.setIcon(new ThemeResource("img/user.png"));
		teachersPanel.getContent().setSizeFull();
			
		
		VerticalLayout vl =  (VerticalLayout) teachersPanel.getContent();
		vl.setMargin(false);
		vl.setExpandRatio(teachersTable, 1.0f);
		vl.setSpacing(false);
		
		addComponent(toolbarteachers);
		addComponent(teachersPanel);
		
		setSizeFull();
		setMargin(false, true, false, true);
		
		setExpandRatio(teachersPanel, 1.0f);
		setSpacing(true);
		
	}
	public void attach() {
		app = (AttendancemanagerApplication)getApplication();
		initLayout();
		teachersTable.addContainerProperty(DatabaseStorage.FULLNAME_PROPERTY, String.class, "");
		teachersTable.addContainerProperty(DatabaseStorage.LOGIN_PROPERTY, String.class, "");
		teachersTable.addContainerProperty(DatabaseStorage.PASSWORD_PROPERTY, String.class, "");
		teachersTable.addContainerProperty(DatabaseStorage.EMAIL_PROPERTY, String.class, "");
		teachersTable.addContainerProperty(DatabaseStorage.COMMENT_PROPERTY, String.class, "");
		teachersTable.addContainerProperty(DatabaseStorage.ADMIN_PROPERTY, String.class, "");

		teachersTable.setColumnHeaders(new String[] { app.locale.getString("GROUPMANAGERLAYOUT_GROUPSTABLE_NAME"),app.locale.getString("LOGINLAYOUT_LOGINFORM_LOGINCAPTION"),app.locale.getString("LOGINLAYOUT_LOGINFORM_PASSWORDCAPTION"),app.locale.getString("GROUPMANAGERLAYOUT_STUDENTGROUPTABLE_EMAIL"),app.locale.getString("GROUPMANAGERLAYOUT_GROUPSTABLE_COMMENT"),app.locale.getString("TEACHERSMANAGERLAYOUT_TEACHERTABLE_ADMINCOLUMN") });
		//app.storage.refreshTable(app.storage.getStudentsQueryContainer(), teachersTable);

		String visiblecolumns[]={DatabaseStorage.FULLNAME_PROPERTY,DatabaseStorage.LOGIN_PROPERTY,DatabaseStorage.EMAIL_PROPERTY,DatabaseStorage.COMMENT_PROPERTY,DatabaseStorage.ADMIN_PROPERTY};
		teachersTable.setVisibleColumns(visiblecolumns);
		teachersTable.setSelectable(true);
		teachersTable.setMultiSelect(false);
		teachersTable.setImmediate(true);

	
		teachersTable.addListener(new Property.ValueChangeListener() {
			
			@Override
			public void valueChange(ValueChangeEvent event) {
				Object value = teachersTable.getValue();
				if (value!=null){
					editButton.setEnabled(true);
					deleteButton.setEnabled(true);
				} else {
					editButton.setEnabled(false);
					deleteButton.setEnabled(false);
					
				}
			}
		});

		newTeacherWindow = new NewTeacherWindow();
		newTeacherWindow.setCaption(app.locale.getString("TEACHERSMANAGERLAYOUT_NEWTEACHERWINDOW_CAPTION"));
		
		editTeachersWindow = new EditTeacherWindow();
		editTeachersWindow.setCaption(app.locale.getString("TEACHERSMANAGERLAYOUT_EDITTEACHERSWINDOW_CAPTION"));
		
		deleteTeachersWindow= new DeleteTeacherWindow();
		deleteTeachersWindow.setCaption(app.locale.getString("TEACHERSMANAGERLAYOUT_DELETETEACHERSWINDOW_CAPTION"));
		
		try {
			importTeachersWindow= new ImportTeacherWindow();
		} catch (SecurityException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (NoSuchMethodException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		importTeachersWindow.setCaption(app.locale.getString("TEACHERSMANAGERLAYOUT_IMPORTTEACHERSWINDOW_CAPTION"));
				
		newButton.addListener(new Button.ClickListener() {
			
			@Override
			public void buttonClick(ClickEvent event) {
				// Open window if not open already
				if(newTeacherWindow.getParent() != null) {
					// Window already open
				} else {
					// open window
					getWindow().addWindow(newTeacherWindow);
				}
			}
		});
			editButton.addListener(new Button.ClickListener() {
			
			@Override
			public void buttonClick(ClickEvent event) {
				// TODO Auto-generated method stub
				Integer idItem=(Integer) teachersTable.getValue();
				Item item=teachersTable.getItem(idItem);
				Integer idTeacher=(Integer) teachersTable.getValue();
				
				String fullname=(String) item.getItemProperty(DatabaseStorage.FULLNAME_PROPERTY).getValue();
				String comment=(String) item.getItemProperty(DatabaseStorage.COMMENT_PROPERTY).getValue();
				String email=(String) item.getItemProperty(DatabaseStorage.EMAIL_PROPERTY).getValue();
				String login=(String) item.getItemProperty(DatabaseStorage.LOGIN_PROPERTY).getValue();
				String password=(String) item.getItemProperty(DatabaseStorage.PASSWORD_PROPERTY).getValue();
				Boolean admin =(Boolean) item.getItemProperty(DatabaseStorage.ADMIN_PROPERTY).getValue().equals("true") ? true : false;
				Teacher teacher = new Teacher(idTeacher,login,password, fullname,   email, comment, admin);
				editTeachersWindow.setTeacher(teacher);
				
				getWindow().addWindow(editTeachersWindow);
			}
		});
			deleteButton.addListener(new Button.ClickListener() {
				
				@Override
				public void buttonClick(ClickEvent event) {
					// TODO Auto-generated method stub
					Integer idItem=(Integer) teachersTable.getValue();
					Item item=teachersTable.getItem(idItem);
					Integer idTeacher=(Integer) teachersTable.getValue();
					String fullname=(String) item.getItemProperty(DatabaseStorage.FULLNAME_PROPERTY).getValue();
					String comment=(String) item.getItemProperty(DatabaseStorage.COMMENT_PROPERTY).getValue();
					String email=(String) item.getItemProperty(DatabaseStorage.EMAIL_PROPERTY).getValue();
					String login=(String) item.getItemProperty(DatabaseStorage.LOGIN_PROPERTY).getValue();
					String password=(String) item.getItemProperty(DatabaseStorage.PASSWORD_PROPERTY).getValue();
					Boolean admin =(Boolean) item.getItemProperty(DatabaseStorage.ADMIN_PROPERTY).getValue().equals("true") ? true : false;
					Teacher teacher = new Teacher(idTeacher,login,password, fullname,   email, comment, admin);
					
					deleteTeachersWindow.setTeacher(teacher);
					getWindow().addWindow(deleteTeachersWindow);
				}
			});
			importButton.addListener(new Button.ClickListener() {
				
				@Override
				public void buttonClick(ClickEvent event) {
					// TODO Auto-generated method stub
					getWindow().addWindow(importTeachersWindow);
				}
			});
		refreshTeachers();
	}
	
	
	
	public void refreshTeachers() {
		app.storage.refreshTable(app.storage.getTeachers(), teachersTable);
	}

	
	/*
	 * Inner classes
	 */
	public class TeacherInfo {

		public String getPasswordcheck() {
			return passwordcheck;
		}
		public void setPasswordcheck(String passwordcheck) {
			this.passwordcheck = passwordcheck;
		}
		public String getPassword() {
			return password;
		}
		public void setPassword(String password) {
			this.password = password;
		}
		public String fullname = "";
		public String password ="";
		public String passwordcheck ="";
		public String login = "";
		public String email = "";
		public String comment = "";
		public Boolean admin = false;
		
		
		public String getFullname() {
			return fullname;
		}
		public void setFullname(String fullname) {
			this.fullname = fullname;
		}
		public String getEmail() {
			return email;
		}
		public void setEmail(String email) {
			this.email = email;
		}
		public String getLogin() {
			return login;
		}
		public void setLogin(String login) {
			this.login = login;
		}
		public String getComment() {
			return comment;
		}
		public void setComment(String comment) {
			this.comment = comment;
		}
		public Boolean getAdmin() {
			return admin;
		}
		public void setAdmin(Boolean admin) {
			this.admin = admin;
		}
		
	}
	private class PasswordValidator extends AbstractStringValidator{
		private TextField pass1;
		private TextField pass2;
		
		public PasswordValidator(String errorMessage,TextField pass1,TextField pass2) {
			this(errorMessage);
			this.pass1=pass1;
			this.pass2=pass2;
		}
	public PasswordValidator(String errorMessage) {
			super(errorMessage);
			// TODO Auto-generated constructor stub
		}

		@Override
		protected boolean isValidString(String value) {
			// TODO Auto-generated method stub
			if (pass1.getValue().equals(pass2.getValue())) {
				return true;
			}else{
				return false;
			}
		}
	}
	private class LoginValidator extends AbstractStringValidator{
		private TextField login;
		private AttendancemanagerApplication app;
		private Integer idTeacher;
		
	public LoginValidator(String errorMessage,TextField login) {
			this(errorMessage);
			this.login=login;
			this.app = (AttendancemanagerApplication)getApplication();
			this.idTeacher=null;
		}
	
	public LoginValidator(String errorMessage,TextField login,Integer idTeacher) {
			this(errorMessage);
			this.login=login;
			this.app = (AttendancemanagerApplication)getApplication();
			this.idTeacher=idTeacher;
		}
	public LoginValidator(String errorMessage) {
			super(errorMessage);
			// TODO Auto-generated constructor stub
		}

		@Override
		protected boolean isValidString(String value) {
			// TODO Auto-generated method stub
			if (idTeacher==null){
				return ! app.storage.loginExists((String)login.getValue());
			}else{
				return ! app.storage.loginExists((String)login.getValue(),idTeacher);
			}
		}
	}
	private class NewTeacherWindow extends Window {
		
		private final Form teacherForm = new Form();
		private final TeacherInfo teacherInfo = new TeacherInfo();
		
		public NewTeacherWindow() {
			
			teacherForm.setValidationVisible(true);
			
			BeanItem<TeacherInfo> teacherBean = new BeanItem<TeacherInfo>(teacherInfo);
			teacherForm.setItemDataSource(teacherBean,Arrays.asList ( "fullname","login","password","passwordcheck","email","comment","admin"));
			
			TextField tf = (TextField)teacherForm.getField("fullname");
			tf.setCaption(app.locale.getString("GROUPMANAGERLAYOUT_GROUPSTABLE_NAME"));
			//tf.setRequiredError(app.locale.getString("TEACHERSMANAGERLAYOUT_NEWTEACHERWINDOW_NAMEREQUIREDERROR"));
			tf.addValidator(new StringLengthValidator(
					app.locale.getString("GROUPMANAGERLAYOUT_NEWGROUPWINDOW_NAMETOOLONGERROR")+ Constants.SHORT_VARCHAR_LENGTH + " " + app.locale.getString("GROUPMANAGERLAYOUT_NEWGROUPWINDOW_CHARS"),1,Constants.SHORT_VARCHAR_LENGTH,false));
			
			tf = (TextField)teacherForm.getField("login");
			tf.setRequired(true);
			tf.setCaption(app.locale.getString("LOGINLAYOUT_LOGINFORM_LOGINCAPTION"));
			tf.addValidator(new LoginValidator(app.locale.getString("TEACHERSMANAGERLAYOUT_NEWTEACHERWINDOW_LOGINVALITORERROR"),(TextField)teacherForm.getField("login")));
			tf.addValidator(new StringLengthValidator(
					app.locale.getString("TEACHERSMANAGERLAYOUT_NEWTEACHERWINDOW_LOGINTOOLONG")+ Constants.SHORT_VARCHAR_LENGTH + " " + app.locale.getString("GROUPMANAGERLAYOUT_NEWGROUPWINDOW_CHARS"),1,Constants.SHORT_VARCHAR_LENGTH,false));
			
			tf = (TextField)teacherForm.getField("password");
			tf.setRequired(true);
			tf.setCaption(app.locale.getString("LOGINLAYOUT_LOGINFORM_PASSWORDCAPTION"));
			tf.addValidator(new StringLengthValidator(
					app.locale.getString("TEACHERSMANAGERLAYOUT_NEWTEACHERWINDOW_PASSWORDTOOLONG")+ Constants.SHORT_VARCHAR_LENGTH + " " + app.locale.getString("GROUPMANAGERLAYOUT_NEWGROUPWINDOW_CHARS"),1,Constants.SHORT_VARCHAR_LENGTH,false));
			
			tf = (TextField)teacherForm.getField("passwordcheck");
			tf.setRequired(true);
			tf.setCaption(app.locale.getString("TEACHERSMANAGERLAYOUT_NEWTEACHERWINDOW_PASSWORDCHECKCAPTION"));
			tf.addValidator(new PasswordValidator(app.locale.getString("TEACHERSMANAGERLAYOUT_NEWTEACHERWINDOW_PASSWORDCHECKERROR"),(TextField)teacherForm.getField("password"),tf));
			tf.addValidator(new StringLengthValidator(
					app.locale.getString("TEACHERSMANAGERLAYOUT_NEWTEACHERWINDOW_PASSWORDTOOLONG")+ Constants.SHORT_VARCHAR_LENGTH + " " + app.locale.getString("GROUPMANAGERLAYOUT_NEWGROUPWINDOW_CHARS"),1,Constants.SHORT_VARCHAR_LENGTH,false));
			
			tf = (TextField)teacherForm.getField("comment");
			tf.setCaption(app.locale.getString("GROUPMANAGERLAYOUT_GROUPSTABLE_COMMENT"));
			tf.addValidator(new StringLengthValidator(
					app.locale.getString("GROUPMANAGERLAYOUT_NEWGROUPWINDOW_COMMENTTOOLONGERROR")+ Constants.SHORT_VARCHAR_LENGTH + " " + app.locale.getString("GROUPMANAGERLAYOUT_NEWGROUPWINDOW_CHARS"),1,Constants.SHORT_VARCHAR_LENGTH,false));

			tf = (TextField)teacherForm.getField("email");
			tf.setCaption(app.locale.getString("GROUPMANAGERLAYOUT_STUDENTGROUPTABLE_EMAIL"));
			tf.addValidator(new EmailValidator(
					app.locale.getString("STUDENTSMANAGERLAYOUT_NEWSTUDENTWINDOW_EMAILVALIDATOR")));
			tf.addValidator(new StringLengthValidator(
					app.locale.getString("STUDENTSMANAGERLAYOUT_NEWSTUDENTWINDOW_EMAILTOOLONGERROR")+ Constants.SHORT_VARCHAR_LENGTH + " " + app.locale.getString("GROUPMANAGERLAYOUT_NEWGROUPWINDOW_CHARS"),1,Constants.SHORT_VARCHAR_LENGTH,false));
			
			CheckBox ch = (CheckBox)teacherForm.getField("admin");
			ch.setCaption(app.locale.getString("TEACHERSMANAGERLAYOUT_TEACHERTABLE_ADMINCOLUMN"));
			ch.setRequired(false);

			teacherForm.setImmediate(true);
		
			Button okButton = new Button(app.locale.getString("GROUPMANAGERLAYOUT_BUTTONOK_CAPTION"));
			okButton.addListener(new Button.ClickListener() {
				
				@Override
				public void buttonClick(ClickEvent event) {
					if (!teacherForm.isValid()) return;
					//Teacher(int id, String login, String password, String fullname, String email, String comment, boolean admin)
					Teacher teacher = new Teacher(0, 
							(String)teacherForm.getField("login").getValue(), 
							(String)app.storage.getPasswordHash((String)teacherForm.getField("password").getValue()),
							(String)teacherForm.getField("fullname").getValue(),
							(String)teacherForm.getField("email").getValue(),
							(String)teacherForm.getField("comment").getValue(),
							(Boolean)teacherForm.getField("admin").getValue());
	
					Integer idnewTeacher=app.storage.addTeacher(teacher);
                    getParent().getWindow().showNotification(
                    		app.locale.getString("GROUPMANAGERLAYOUT_TOOLTIP_MESSAGE"),
                    		app.locale.getString("TEACHERSMANAGERLAYOUT_TOOLTIP_TEACHERADDED")+ " " +(String)teacherForm.getField("fullname").getValue(),
                            Notification.TYPE_TRAY_NOTIFICATION);
					setValuesEmpty();
					refreshTeachers();
					teachersTable.setValue(idnewTeacher);
                    close();
				}
			});
			okButton.setIcon(new ThemeResource("img/12-em-check.png"));
			
			Button cancelButton = new Button(app.locale.getString("GROUPMANAGERLAYOUT_BUTTONCANCEL_CAPTION"));
			cancelButton.addListener(new Button.ClickListener() {
				
				@Override
				public void buttonClick(ClickEvent event) {   
					setValuesEmpty();
                	close();
				}
			});
			cancelButton.setIcon(new ThemeResource("img/12-em-cross.png"));
			HorizontalLayout hl = new HorizontalLayout();
			teacherForm.getFooter().addComponent(hl);
			hl.setSpacing(true);
			hl.addComponent(okButton);
			hl.addComponent(cancelButton);
			
    		addComponent(teacherForm);
    		
    		VerticalLayout layout = (VerticalLayout) getContent();
    		layout.setSpacing(true);
    		layout.setMargin(true);
    		layout.setSizeUndefined(); // We want the window to has automatic size !!!
    		
    		setModal(true);
    		setResizable(false);
		}
		private void setValuesEmpty(){
			teacherForm.getField("fullname").setValue("");
			teacherForm.getField("login").setValue("");
			teacherForm.getField("password").setValue("");
			teacherForm.getField("passwordcheck").setValue("");
			teacherForm.getField("admin").setValue(false);
			teacherForm.getField("comment").setValue("");
			teacherForm.getField("email").setValue("");
		}
	}
    private class EditTeacherWindow extends Window {
		
		private final Form teacherForm = new Form();
		private final TeacherInfo teacherInfo = new TeacherInfo();
		private Teacher teacher;
		
		public EditTeacherWindow() {
			
			teacherForm.setValidationVisible(true);
			
			BeanItem<TeacherInfo> teacherBean = new BeanItem<TeacherInfo>(teacherInfo);
			teacherForm.setItemDataSource(teacherBean,Arrays.asList ( "fullname","login","password","passwordcheck","email","comment","admin"));
			
			TextField tf = (TextField)teacherForm.getField("fullname");
			tf.setCaption(app.locale.getString("GROUPMANAGERLAYOUT_GROUPSTABLE_NAME"));
			//tf.setRequiredError(app.locale.getString("TEACHERSMANAGERLAYOUT_NEWTEACHERWINDOW_NAMEREQUIREDERROR"));
			tf.addValidator(new StringLengthValidator(
					app.locale.getString("GROUPMANAGERLAYOUT_NEWGROUPWINDOW_NAMETOOLONGERROR")+ Constants.SHORT_VARCHAR_LENGTH + " " + app.locale.getString("GROUPMANAGERLAYOUT_NEWGROUPWINDOW_CHARS"),1,Constants.SHORT_VARCHAR_LENGTH,false));
		    
			tf = (TextField)teacherForm.getField("login");
			tf.setRequired(true);
			tf.setCaption(app.locale.getString("LOGINLAYOUT_LOGINFORM_LOGINCAPTION"));
			//tf.addValidator(new LoginValidator("You must use a diferent login. This is in use",(TextField)teacherForm.getField("login"),(Integer)teachersTable.getValue()));
			tf.addValidator(new StringLengthValidator(
					app.locale.getString("TEACHERSMANAGERLAYOUT_NEWTEACHERWINDOW_LOGINTOOLONG")+ Constants.SHORT_VARCHAR_LENGTH + " " + app.locale.getString("GROUPMANAGERLAYOUT_NEWGROUPWINDOW_CHARS"),1,Constants.SHORT_VARCHAR_LENGTH,false));
			
			tf = (TextField)teacherForm.getField("password");
			tf.setRequired(true);
			tf.setCaption(app.locale.getString("LOGINLAYOUT_LOGINFORM_PASSWORDCAPTION"));
			tf.setSecret(true);
			tf.addValidator(new StringLengthValidator(
					app.locale.getString("TEACHERSMANAGERLAYOUT_NEWTEACHERWINDOW_PASSWORDTOOLONG")+ Constants.SHORT_VARCHAR_LENGTH + " " + app.locale.getString("GROUPMANAGERLAYOUT_NEWGROUPWINDOW_CHARS"),1,Constants.SHORT_VARCHAR_LENGTH,false));
			
			tf = (TextField)teacherForm.getField("passwordcheck");
			tf.setRequired(true);
			tf.setCaption(app.locale.getString("TEACHERSMANAGERLAYOUT_NEWTEACHERWINDOW_PASSWORDCHECKCAPTION"));
			tf.setSecret(true);
			tf.addValidator(new PasswordValidator(app.locale.getString("TEACHERSMANAGERLAYOUT_NEWTEACHERWINDOW_PASSWORDCHECKERROR"),(TextField)teacherForm.getField("password"),tf));
			tf.addValidator(new StringLengthValidator(
					app.locale.getString("TEACHERSMANAGERLAYOUT_NEWTEACHERWINDOW_PASSWORDTOOLONG")+ Constants.SHORT_VARCHAR_LENGTH + " " + app.locale.getString("GROUPMANAGERLAYOUT_NEWGROUPWINDOW_CHARS"),1,Constants.SHORT_VARCHAR_LENGTH,false));
			
			tf = (TextField)teacherForm.getField("comment");
			tf.setCaption(app.locale.getString("GROUPMANAGERLAYOUT_GROUPSTABLE_COMMENT"));
			tf.addValidator(new StringLengthValidator(
					app.locale.getString("GROUPMANAGERLAYOUT_NEWGROUPWINDOW_COMMENTTOOLONGERROR")+ Constants.SHORT_VARCHAR_LENGTH + " " + app.locale.getString("GROUPMANAGERLAYOUT_NEWGROUPWINDOW_CHARS"),1,Constants.SHORT_VARCHAR_LENGTH,false));

			tf = (TextField)teacherForm.getField("email");
			tf.setCaption(app.locale.getString("GROUPMANAGERLAYOUT_STUDENTGROUPTABLE_EMAIL"));
			tf.addValidator(new EmailValidator(
					app.locale.getString("STUDENTSMANAGERLAYOUT_NEWSTUDENTWINDOW_EMAILVALIDATOR")));
			tf.addValidator(new StringLengthValidator(
					app.locale.getString("STUDENTSMANAGERLAYOUT_NEWSTUDENTWINDOW_EMAILTOOLONGERROR")+ Constants.SHORT_VARCHAR_LENGTH + " " + app.locale.getString("GROUPMANAGERLAYOUT_NEWGROUPWINDOW_CHARS"),1,Constants.SHORT_VARCHAR_LENGTH,false));
			
			CheckBox ch = (CheckBox)teacherForm.getField("admin");
			ch.setCaption(app.locale.getString("TEACHERSMANAGERLAYOUT_TEACHERTABLE_ADMINCOLUMN"));
			ch.setRequired(false);
			
			teacherForm.setImmediate(true);
		
			Button okButton = new Button(app.locale.getString("GROUPMANAGERLAYOUT_BUTTONOK_CAPTION"));
			okButton.addListener(new Button.ClickListener() {
				
				@Override
				public void buttonClick(ClickEvent event) {
					if (!teacherForm.isValid()) return;
					Teacher editedteacher = new Teacher(teacher.getID(), 
							(String)teacherForm.getField("login").getValue(), 
							(String)app.storage.getPasswordHash((String)teacherForm.getField("password").getValue()),
							(String)teacherForm.getField("fullname").getValue(),
							(String)teacherForm.getField("email").getValue(),
							(String)teacherForm.getField("comment").getValue(),
							(Boolean)teacherForm.getField("admin").getValue());

					if (teacherForm.getField("password").getValue().equals(teacher.getPassword())){
						//No ha cambiado el password
						app.storage.updateTeacher(editedteacher,false);
					}else{
						////Ha cambiado el password
						app.storage.updateTeacher(editedteacher,true);
					}
                    getParent().getWindow().showNotification(
                    		app.locale.getString("GROUPMANAGERLAYOUT_TOOLTIP_MESSAGE"),
                    		app.locale.getString("TEACHERSMANAGERLAYOUT_TOOLTIP_TEACHEREDITED")+ " " +(String)teacherForm.getField("fullname").getValue(),
                            Notification.TYPE_TRAY_NOTIFICATION);
					setValuesEmpty();
					refreshTeachers();
					teachersTable.setValue(teacher.getID());
                    close();
				}
			});
			okButton.setIcon(new ThemeResource("img/12-em-check.png"));
			
			Button cancelButton = new Button(app.locale.getString("GROUPMANAGERLAYOUT_BUTTONCANCEL_CAPTION"));
			cancelButton.addListener(new Button.ClickListener() {
				
				@Override
				public void buttonClick(ClickEvent event) {   
					setValuesEmpty();
                	close();
				}
			});
			cancelButton.setIcon(new ThemeResource("img/12-em-cross.png"));
			HorizontalLayout hl = new HorizontalLayout();
			teacherForm.getFooter().addComponent(hl);
			hl.setSpacing(true);
			hl.addComponent(okButton);
			hl.addComponent(cancelButton);
			
    		addComponent(teacherForm);
    		
    		VerticalLayout layout = (VerticalLayout) getContent();
    		layout.setSpacing(true);
    		layout.setMargin(true);
    		layout.setSizeUndefined(); // We want the window to has automatic size !!!
    		
    		setModal(true);
    		setResizable(false);
		}
		private void setValuesEmpty(){
			teacherForm.getField("fullname").setValue("");
			teacherForm.getField("login").setValue("");
			teacherForm.getField("password").setValue("");
			teacherForm.getField("passwordcheck").setValue("");
			teacherForm.getField("admin").setValue(false);
			teacherForm.getField("comment").setValue("");
			teacherForm.getField("email").setValue("");
		}
		public void setTeacher(Teacher teacher){
			this.teacher=teacher;
			teacherForm.getField("fullname").setValue(teacher.getFullname());
			teacherForm.getField("comment").setValue(teacher.getComment());
			teacherForm.getField("email").setValue(teacher.getEmail());
			teacherForm.getField("password").setValue(teacher.getPassword());
			teacherForm.getField("passwordcheck").setValue(teacher.getPassword());
			teacherForm.getField("login").setValue(teacher.getLogin());
			teacherForm.getField("admin").setValue(teacher.getAdmin());
		}
	}
	private class DeleteTeacherWindow extends Window {
		
		private Teacher teacher;
		private Label messageLabel;

		public DeleteTeacherWindow() {
			Embedded warningEmbedded = new Embedded(null,new ThemeResource("img/Warning_icon.png"));
			 
			messageLabel = new Label();
			String messageString = app.locale.getString("TEACHERSMANAGERLAYOUT_DELETEWINDOW_MESSAGE");
			
			messageLabel.setValue(messageString);
			messageLabel.setContentMode(Label.CONTENT_XHTML);
			
			HorizontalLayout warningLayout = new HorizontalLayout();
			warningLayout.setSpacing(true);
			warningLayout.addComponent(warningEmbedded);
			warningLayout.addComponent(messageLabel);
			warningLayout.setSizeFull();
			
			Button okButton = new Button(app.locale.getString("GROUPMANAGERLAYOUT_BUTTONOK_CAPTION"));
			okButton.addListener(new Button.ClickListener() {
				
				@Override
				public void buttonClick(ClickEvent event) {
					app.storage.deleteTeacher(teacher.getID());
                    getParent().getWindow().showNotification(
                    		app.locale.getString("GROUPMANAGERLAYOUT_TOOLTIP_MESSAGE"),
                    		app.locale.getString("TEACHERSMANAGERLAYOUT_TOOLTIP_TEACHERDELETED")+ " " +(String)teacher.getFullname(),
                    		Notification.TYPE_TRAY_NOTIFICATION);
					refreshTeachers();
					close();
				}
			});
			okButton.setIcon(new ThemeResource("img/12-em-check.png"));
			
			Button cancelButton = new Button(app.locale.getString("GROUPMANAGERLAYOUT_BUTTONCANCEL_CAPTION"));
			cancelButton.addListener(new Button.ClickListener() {
				
				@Override
				public void buttonClick(ClickEvent event) {
					close();
				}
			});
			cancelButton.setIcon(new ThemeResource("img/12-em-cross.png"));
			HorizontalLayout hl = new HorizontalLayout();
			hl.setSpacing(true);
			hl.setMargin(true);
			hl.addComponent(okButton);
			hl.addComponent(cancelButton);
			hl.setSizeFull();
			addComponent(warningLayout);
    		addComponent(hl);
    		
    		VerticalLayout layout = (VerticalLayout) getContent();
    		layout.setSpacing(true);
    		layout.setMargin(true);
    		layout.setSizeUndefined(); // We want the window to has automatic size !!!
    		
    		setModal(true);
    		setResizable(false);
		}
		public void setTeacher(Teacher teacher){
			this.teacher=teacher;
			
		}
	}	
	
	/**
	 * Ventana emergente de importación de datos en formato csv.
	 * 
	 * @author apalacios
	 *
	 */
	private class ImportTeacherWindow extends Window implements Upload.SucceededListener, Upload.FailedListener{
		
		private CSVUploader csvuploader = new CSVUploader(Teacher.class.getConstructor(String.class, String.class, String.class, String.class), Teacher.getBasicProperties());
		private Upload upload=new Upload(app.locale.getString("GROUPMANAGERLAYOUT_UPLOAD_CAPTION"),csvuploader);
		
		/**
		 * Constructor de la ventana de importación de csv. Crea un objeto Upload y lo conecta con el CSVUploader.
		 * @throws SecurityException
		 * @throws NoSuchMethodException
		 */
		public ImportTeacherWindow() throws SecurityException, NoSuchMethodException {

			String alert = "<p><br/>Asegúrese de que la estructura de los registros del CSV sea la siguiente:</p>" + 
			"<table border=\"1\" align=\"center\">" +
			"<tr>" + "<th> login</th>" + "<th> password</th>" + "<th> fullname</th>" + "<th> email</th>" + "</tr>" +
			"<tr>" + "<th> \"prof1\"</th>" + "<th> \"prof1\"</th>" + "<th> \"prof1\"</th>" + "<th> \"prof1@mail.com\"</th>" +
			"</tr>" + "</table>";
			
			csvuploader.setAlert(alert);
			
			addComponent(upload);
			upload.setButtonCaption(app.locale.getString("GROUPMANAGERLAYOUT_UPLOADBUTTON_CAPTION"));
			
			upload.addListener((Upload.FinishedListener)csvuploader);
			upload.addListener((Upload.SucceededListener)this);
			upload.addListener((Upload.FailedListener)this);
						
			Button cancelButton = new Button(app.locale.getString("GROUPMANAGERLAYOUT_BUTTONCANCEL_CAPTION"));
			cancelButton.addListener(new Button.ClickListener() {
				
				@Override
				public void buttonClick(ClickEvent event) {
					close();
				}
			});
			cancelButton.setIcon(new ThemeResource("img/12-em-cross.png"));
			HorizontalLayout hl = new HorizontalLayout();
			hl.setSpacing(true);
			hl.setMargin(true);
			hl.addComponent(cancelButton);
    		addComponent(hl);
    		
    		VerticalLayout layout = (VerticalLayout) getContent();
    		layout.setSpacing(true);
    		layout.setMargin(true);
    		layout.setSizeUndefined(); // We want the window to has automatic size !!!
    		
    		setModal(true);
    		setResizable(false);
		}

		/**
		 * Evento de carga correcta. Muestra una notificación con el número de registros correctos y fallidos.
		 */
		@Override		
		public void uploadSucceeded(SucceededEvent event) {
			refreshTeachers();
			close();
			System.out.println("Profesores cargados !!!");
			
			String finishMessage = app.locale.getString("GROUPMANAGERLAYOUT_UPLOADSUCCESS_TAGMESSAGE")+" "
				+ csvuploader.successfulRecordsCounter + " "
				+ app.locale.getString("GROUPMANAGERLAYOUT_UPLOADSUCCESSOK_MESSAGE")
				+ app.locale.getString("GROUPMANAGERLAYOUT_UPLOADSUCCESS_TAGMESSAGE")+" "
				+ csvuploader.failedRecordsCounter + " "
				+ app.locale.getString("GROUPMANAGERLAYOUT_UPLOADSUCCESSNOK_MESSAGE");
			
			Label finishLabel = csvuploader.getLabelFromString(finishMessage);
			AttendancemanagerApplication.adminWindow.showNotification("CARGA FINALIZADA",(String) finishLabel.getValue(), Notification.TYPE_TRAY_NOTIFICATION);			
		}

		/**
		 * Evento de carga fallida. Si la carga falla, muestra una alerta de error.
		 */
		@Override
		public void uploadFailed(FailedEvent event) {
			System.out.println("Ohhhh, no he podido insertar los profesores !!!");
			
			String errorMessage = app.locale.getString("GROUPMANAGERLAYOUT_UPLOADFAILED_ERRORSTRING");
			Label errorLabel = csvuploader.getLabelFromString(errorMessage);
			AttendancemanagerApplication.adminWindow.showNotification("ERROR",(String) errorLabel.getValue(), Notification.TYPE_ERROR_MESSAGE);
		}
	}
}
