/* 
 * Copyright 2009 IT Mill Ltd.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */

package com.vaadin.terminal;

import java.io.Serializable;
import java.net.URL;

/**
 * Interface implemented by all the classes capable of handling URI:s.
 * 
 * <p>
 * <code>URIHandler</code> can provide <code>DownloadStream</code> for
 * transferring data for client.
 * </p>
 * 
 * @author IT Mill Ltd.
 * @version
 * 6.2.3
 * @since 3.0
 */
public interface URIHandler extends Serializable {

    /**
     * Handles a given relative URI. If the URI handling wants to emit a
     * downloadable stream it can return download stream object. If no emitting
     * stream is necessary, null should be returned instead.
     * 
     * @param context
     *            the URl.
     * @param relativeUri
     *            the relative uri.
     * @return the download stream object.
     */
    public DownloadStream handleURI(URL context, String relativeUri);

    /**
     * URIHandler error event.
     */
    public interface ErrorEvent extends Terminal.ErrorEvent {

        /**
         * Gets the source URIHandler.
         * 
         * @return the URIHandler.
         */
        public URIHandler getURIHandler();

    }
}
